# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    The version of the OpenAPI document: 1.113
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import
import re  # noqa: F401
import sys  # noqa: F401

import six  # noqa: F401
import nulltype  # noqa: F401

from onshape_client.oas.model_utils import (  # noqa: F401
    ModelComposed,
    ModelNormal,
    ModelSimple,
    date,
    datetime,
    file_type,
    int,
    none_type,
    str,
    validate_get_composed_info,
)

try:
    from onshape_client.oas.models import bt_export_tessellated_body3398
except ImportError:
    bt_export_tessellated_body3398 = sys.modules[
        "onshape_client.oas.models.bt_export_tessellated_body3398"
    ]


class BTExportTessellatedEdgesResponse327(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ("error_enum",): {
            "NO_ERROR": "NO_ERROR",
            "UNKNOWN_OPERATION": "UNKNOWN_OPERATION",
            "TOO_MANY_ENTITIES_SELECTED": "TOO_MANY_ENTITIES_SELECTED",
            "POINTS_COINCIDENT": "POINTS_COINCIDENT",
            "NO_TRANSLATION_DIRECTION": "NO_TRANSLATION_DIRECTION",
            "NO_ROTATION_AXIS": "NO_ROTATION_AXIS",
            "NO_TANGENT_PLANE": "NO_TANGENT_PLANE",
            "NO_TANGENT_LINE": "NO_TANGENT_LINE",
            "INVALID_INPUT": "INVALID_INPUT",
            "CANNOT_RESOLVE_ENTITIES": "CANNOT_RESOLVE_ENTITIES",
            "CANNOT_EVALUATE_VERTEX": "CANNOT_EVALUATE_VERTEX",
            "CANNOT_RESOLVE_PLANE": "CANNOT_RESOLVE_PLANE",
            "CANNOT_COMPUTE_BBOX": "CANNOT_COMPUTE_BBOX",
            "CANNOT_BE_EMPTY": "CANNOT_BE_EMPTY",
            "CACHE_WRITE_FAILED": "CACHE_WRITE_FAILED",
            "CACHE_READ_FAILED": "CACHE_READ_FAILED",
            "HLR_FAILED": "HLR_FAILED",
            "BAD_GEOMETRY": "BAD_GEOMETRY",
            "INVALID_RESULT": "INVALID_RESULT",
            "MISSING_EXT_REF": "MISSING_EXT_REF",
            "READ_FAILED": "READ_FAILED",
            "WRITE_FAILED": "WRITE_FAILED",
            "WRONG_TYPE": "WRONG_TYPE",
            "TANGENT_PROPAGATION_FAILED": "TANGENT_PROPAGATION_FAILED",
            "REGEN_ERROR": "REGEN_ERROR",
            "COULD_NOT_COMPUTE_TRANSFORM": "COULD_NOT_COMPUTE_TRANSFORM",
            "MATE_INVALID_MATE": "MATE_INVALID_MATE",
            "MATECONNECTOR_INVALID_MATE": "MATECONNECTOR_INVALID_MATE",
            "MATE_TWO_MATECONNECTORS_NEEDED": "MATE_TWO_MATECONNECTORS_NEEDED",
            "MATECONNECTORS_ON_SAME_OCCURRENCE": "MATECONNECTORS_ON_SAME_OCCURRENCE",
            "MATE_OVERDEFINED": "MATE_OVERDEFINED",
            "MATE_INCONSISTENT": "MATE_INCONSISTENT",
            "BOOLEAN_NEED_ONE_SOLID": "BOOLEAN_NEED_ONE_SOLID",
            "BOOLEAN_INVALID": "BOOLEAN_INVALID",
            "BOOLEAN_INTERSECT_FAIL": "BOOLEAN_INTERSECT_FAIL",
            "BOOLEAN_SAME_INPUT": "BOOLEAN_SAME_INPUT",
            "BOOLEAN_BAD_INPUT": "BOOLEAN_BAD_INPUT",
            "BOOLEAN_UNION_NO_OP": "BOOLEAN_UNION_NO_OP",
            "BOOLEAN_INTERSECT_NO_OP": "BOOLEAN_INTERSECT_NO_OP",
            "BOOLEAN_SUBTRACT_NO_OP": "BOOLEAN_SUBTRACT_NO_OP",
            "CPLANE_INPUT_MIDPLANE": "CPLANE_INPUT_MIDPLANE",
            "CPLANE_INPUT_OFFSET_PLANE": "CPLANE_INPUT_OFFSET_PLANE",
            "CPLANE_INPUT_POINT_PLANE": "CPLANE_INPUT_POINT_PLANE",
            "CPLANE_INPUT_LINE_ANGLE": "CPLANE_INPUT_LINE_ANGLE",
            "CPLANE_INPUT_POINT_LINE": "CPLANE_INPUT_POINT_LINE",
            "CPLANE_INPUT_THREE_POINT": "CPLANE_INPUT_THREE_POINT",
            "CPLANE_FAILED": "CPLANE_FAILED",
            "DRAFT_NO_NEUTRAL_PLANE": "DRAFT_NO_NEUTRAL_PLANE",
            "DRAFT_NO_DRAFT_FACE": "DRAFT_NO_DRAFT_FACE",
            "DRAFT_FAILED": "DRAFT_FAILED",
            "EXTRUDE_INVALID_REF_FACE": "EXTRUDE_INVALID_REF_FACE",
            "EXTRUDE_INVALID_REF_SURFACE": "EXTRUDE_INVALID_REF_SURFACE",
            "EXTRUDE_FAILED": "EXTRUDE_FAILED",
            "EXTRUDE_NO_DIRECTION": "EXTRUDE_NO_DIRECTION",
            "EXTRUDE_INVALID_ENTITIES": "EXTRUDE_INVALID_ENTITIES",
            "PATTERN_INPUT_TOO_MANY_INSTANCES": "PATTERN_INPUT_TOO_MANY_INSTANCES",
            "PATTERN_INPUT_TOO_FEW_INSTANCES": "PATTERN_INPUT_TOO_FEW_INSTANCES",
            "PATTERN_FACE_FAILED": "PATTERN_FACE_FAILED",
            "PATTERN_NOT_ON_BODY": "PATTERN_NOT_ON_BODY",
            "PATTERN_BODY_FAILED": "PATTERN_BODY_FAILED",
            "TRANSFORM_TRANSLATE_INPUT": "TRANSFORM_TRANSLATE_INPUT",
            "TRANSFORM_TRANSLATE_BY_DISTANCE_INPUT": "TRANSFORM_TRANSLATE_BY_DISTANCE_INPUT",
            "TRANSFORM_FAILED": "TRANSFORM_FAILED",
            "SHELL_FAILED": "SHELL_FAILED",
            "EDGEBLEND_SMOOTH": "EDGEBLEND_SMOOTH",
            "EDGEBLEND_FAILED": "EDGEBLEND_FAILED",
            "DIRECT_EDIT_WRONG_CONCENTRIC": "DIRECT_EDIT_WRONG_CONCENTRIC",
            "DIRECT_EDIT_WRONG_EQ_RADIUS": "DIRECT_EDIT_WRONG_EQ_RADIUS",
            "DIRECT_EDIT_NO_FILLET_FACES": "DIRECT_EDIT_NO_FILLET_FACES",
            "DIRECT_EDIT_NO_OFFSET": "DIRECT_EDIT_NO_OFFSET",
            "DIRECT_EDIT_CONSTRAIN_FACE_FAILED": "DIRECT_EDIT_CONSTRAIN_FACE_FAILED",
            "DIRECT_EDIT_REPLACE_FACE_FAILED": "DIRECT_EDIT_REPLACE_FACE_FAILED",
            "DIRECT_EDIT_DELETE_FACE_FAILED": "DIRECT_EDIT_DELETE_FACE_FAILED",
            "DIRECT_EDIT_MODIFY_FILLET_FAILED": "DIRECT_EDIT_MODIFY_FILLET_FAILED",
            "DIRECT_EDIT_MODIFY_FACE_FAILED": "DIRECT_EDIT_MODIFY_FACE_FAILED",
            "DIRECT_EDIT_MOVE_FACE_FAILED": "DIRECT_EDIT_MOVE_FACE_FAILED",
            "DIRECT_EDIT_OFFSET_FACE_FAILED": "DIRECT_EDIT_OFFSET_FACE_FAILED",
            "IMPORT_PART_FAILED": "IMPORT_PART_FAILED",
            "IMPORT_ASSEMBLY_FAILED": "IMPORT_ASSEMBLY_FAILED",
            "IMPRINT_FAILED": "IMPRINT_FAILED",
            "REVOLVE_FAILED": "REVOLVE_FAILED",
            "REVOLVE_2ND_DIR_FAILED": "REVOLVE_2ND_DIR_FAILED",
            "REVOLVE_NOT_PLANAR": "REVOLVE_NOT_PLANAR",
            "REVOLVE_PERPENDICULAR": "REVOLVE_PERPENDICULAR",
            "REVOLVE_INVALID_ENTITIES": "REVOLVE_INVALID_ENTITIES",
            "SPLIT_FAILED": "SPLIT_FAILED",
            "SPLIT_INVALID_INPUT": "SPLIT_INVALID_INPUT",
            "SWEEP_INVALID_PATH": "SWEEP_INVALID_PATH",
            "SWEEP_FAILED": "SWEEP_FAILED",
            "SWEEP_PATH_FAILED": "SWEEP_PATH_FAILED",
            "SWEEP_PROFILE_FAILED": "SWEEP_PROFILE_FAILED",
            "WIRE_CREATION_FAILED": "WIRE_CREATION_FAILED",
            "SKETCH_NO_PLANE": "SKETCH_NO_PLANE",
            "SKETCH_INPUT_INVALID": "SKETCH_INPUT_INVALID",
            "SKETCH_NOT_ACTIVE": "SKETCH_NOT_ACTIVE",
            "SKETCH_SOLVER_NOT_INITIALIZED": "SKETCH_SOLVER_NOT_INITIALIZED",
            "SKETCH_EVALUATION_FAILED": "SKETCH_EVALUATION_FAILED",
            "SKETCH_MODIFICATION_FAILED": "SKETCH_MODIFICATION_FAILED",
            "SKETCH_UPDATE_FAILED": "SKETCH_UPDATE_FAILED",
            "SKETCH_SOLVE_FAILED": "SKETCH_SOLVE_FAILED",
            "SKETCH_ADD_CONSTRAINT_FAILED": "SKETCH_ADD_CONSTRAINT_FAILED",
            "SKETCH_ADD_DIMENSION_FAILED": "SKETCH_ADD_DIMENSION_FAILED",
            "SKETCH_POSITION_DIMENSION_FAILED": "SKETCH_POSITION_DIMENSION_FAILED",
            "SKETCH_CONSTRAINT_NEEDS_SKETCH_ENTITY": "SKETCH_CONSTRAINT_NEEDS_SKETCH_ENTITY",
            "SKETCH_CONSTRAINT_UNKNOWN": "SKETCH_CONSTRAINT_UNKNOWN",
            "SKETCH_MISSING_ENTITY": "SKETCH_MISSING_ENTITY",
            "SKETCH_FILLET_INVALID_POINT": "SKETCH_FILLET_INVALID_POINT",
            "SKETCH_FILLET_PARALLEL": "SKETCH_FILLET_PARALLEL",
            "SKETCH_FILLET_FAIL": "SKETCH_FILLET_FAIL",
            "SKETCH_USE_FAILED": "SKETCH_USE_FAILED",
            "SKETCH_USE_PARTIAL": "SKETCH_USE_PARTIAL",
            "SKETCH_SPLINE_FAILED": "SKETCH_SPLINE_FAILED",
            "SKETCH_BAD_SPLINE": "SKETCH_BAD_SPLINE",
            "SKETCH_DRAG_ERROR": "SKETCH_DRAG_ERROR",
            "SKETCH_PROJ_FAILED": "SKETCH_PROJ_FAILED",
            "SKETCH_PROJ_PARTIAL": "SKETCH_PROJ_PARTIAL",
            "SKETCH_TANGENT_ARC_FAILED": "SKETCH_TANGENT_ARC_FAILED",
            "SKETCH_TANGENT_NOT_FOUND": "SKETCH_TANGENT_NOT_FOUND",
            "SKETCH_OFFSET_FAILED": "SKETCH_OFFSET_FAILED",
            "SKETCH_OFFSET_DISTANCE": "SKETCH_OFFSET_DISTANCE",
            "SKETCH_TRIM_FAILED": "SKETCH_TRIM_FAILED",
            "SKETCH_INFERENCE_FAILED": "SKETCH_INFERENCE_FAILED",
            "SKETCH_MODIFY_DIM_FAILED": "SKETCH_MODIFY_DIM_FAILED",
            "SKETCH_DRAG_NO_SKETCH": "SKETCH_DRAG_NO_SKETCH",
            "SKETCH_INFER_DIM_FAILED": "SKETCH_INFER_DIM_FAILED",
            "SKETCH_DELETE_PTS_FAILED": "SKETCH_DELETE_PTS_FAILED",
            "SKETCH_DELETE_FAILED": "SKETCH_DELETE_FAILED",
            "SKETCH_ARC_FAILED": "SKETCH_ARC_FAILED",
            "SKETCH_LINE_FAILED": "SKETCH_LINE_FAILED",
            "SKETCH_CIRCLE_FAILED": "SKETCH_CIRCLE_FAILED",
            "SKETCH_RECTANGLE_FAILED": "SKETCH_RECTANGLE_FAILED",
            "SKETCH_TANGENT_ARC_INVALID_START": "SKETCH_TANGENT_ARC_INVALID_START",
            "SKETCH_CONSTRUCTION_POINT_FAILED": "SKETCH_CONSTRUCTION_POINT_FAILED",
            "SYS_INTERNAL_DESERIALIZATION": "SYS_INTERNAL_DESERIALIZATION",
            "SYS_SERVER_EXCEPTION": "SYS_SERVER_EXCEPTION",
            "SYS_ERROR_REGEN": "SYS_ERROR_REGEN",
            "SYS_ERROR_MESSAGING": "SYS_ERROR_MESSAGING",
            "CANNOT_RESOLVE_ELEMENT": "CANNOT_RESOLVE_ELEMENT",
            "NOTHING_SELECTED": "NOTHING_SELECTED",
            "SKETCH_ANGLE_TWO_LINES": "SKETCH_ANGLE_TWO_LINES",
            "SKETCH_DIMENSION_DIFF_ENTITIES": "SKETCH_DIMENSION_DIFF_ENTITIES",
            "SKETCH_CONSTRAINT_DIFF_ENTITIES": "SKETCH_CONSTRAINT_DIFF_ENTITIES",
            "SKETCH_CONSTRAINT_TWO_ENTITIES": "SKETCH_CONSTRAINT_TWO_ENTITIES",
            "SKETCH_DIMENSION_TWO_ENTITIES": "SKETCH_DIMENSION_TWO_ENTITIES",
            "SKETCH_COINCIDENT_FAILED": "SKETCH_COINCIDENT_FAILED",
            "SKETCH_COINCIDENT_INPUT_ERROR": "SKETCH_COINCIDENT_INPUT_ERROR",
            "SKETCH_COINCIDENT_DIFF_POINTS": "SKETCH_COINCIDENT_DIFF_POINTS",
            "SKETCH_CONCENTRIC_INPUT_ERROR": "SKETCH_CONCENTRIC_INPUT_ERROR",
            "SKETCH_CONCENTRIC_FAILED": "SKETCH_CONCENTRIC_FAILED",
            "SKETCH_EQUAL_INPUT_ERROR": "SKETCH_EQUAL_INPUT_ERROR",
            "SKETCH_EQUAL_NO_ENDS": "SKETCH_EQUAL_NO_ENDS",
            "SKETCH_EQUAL_FAILED": "SKETCH_EQUAL_FAILED",
            "SKETCH_FIX_ONE_ENT": "SKETCH_FIX_ONE_ENT",
            "SKETCH_FIX_FAILED": "SKETCH_FIX_FAILED",
            "SKETCH_DIR_INTERNAL": "SKETCH_DIR_INTERNAL",
            "SKETCH_DIR_INPUT": "SKETCH_DIR_INPUT",
            "SKETCH_HORIZONTAL_FAILED": "SKETCH_HORIZONTAL_FAILED",
            "SKETCH_VERTICAL_FAILED": "SKETCH_VERTICAL_FAILED",
            "SKETCH_OFFSET_CONSTRAINT_FAILED": "SKETCH_OFFSET_CONSTRAINT_FAILED",
            "SKETCH_PARALLEL_CONSTRAINT_FAILED": "SKETCH_PARALLEL_CONSTRAINT_FAILED",
            "SKETCH_PARALLEL_INPUT_ERROR": "SKETCH_PARALLEL_INPUT_ERROR",
            "SKETCH_DIMENSION_INPUT_ERROR": "SKETCH_DIMENSION_INPUT_ERROR",
            "SKETCH_DIMENSION_DIST_ERROR": "SKETCH_DIMENSION_DIST_ERROR",
            "SKETCH_DIMENSION_FAILED": "SKETCH_DIMENSION_FAILED",
            "SKETCH_NORMAL_NEED_LINE": "SKETCH_NORMAL_NEED_LINE",
            "SKETCH_NORMAL_INPUT_ERROR": "SKETCH_NORMAL_INPUT_ERROR",
            "SKETCH_NORMAL_INPUT_NEEDED": "SKETCH_NORMAL_INPUT_NEEDED",
            "SKETCH_CANNOT_SPLIT_INTO_GROUPS": "SKETCH_CANNOT_SPLIT_INTO_GROUPS",
            "SKETCH_OFFSET_BAD_PAIR": "SKETCH_OFFSET_BAD_PAIR",
            "SKETCH_OFFSET_INPUT_ERROR": "SKETCH_OFFSET_INPUT_ERROR",
            "SKETCH_MIDPOINT_INPUT_ERROR": "SKETCH_MIDPOINT_INPUT_ERROR",
            "SKETCH_MIDPOINT_NEED_POINT": "SKETCH_MIDPOINT_NEED_POINT",
            "SKETCH_MIDPOINT_NEED_DIFF_POINT": "SKETCH_MIDPOINT_NEED_DIFF_POINT",
            "SKETCH_MIDPOINT_MISSING_ENDS": "SKETCH_MIDPOINT_MISSING_ENDS",
            "SKETCH_MIDPOINT_MISSING_PTS": "SKETCH_MIDPOINT_MISSING_PTS",
            "SKETCH_MIDPOINT_NO_INTERNAL_LINE": "SKETCH_MIDPOINT_NO_INTERNAL_LINE",
            "SKETCH_MIDPOINT_NO_COINCIDENT": "SKETCH_MIDPOINT_NO_COINCIDENT",
            "SKETCH_MIDPOINT_FAILED": "SKETCH_MIDPOINT_FAILED",
            "SKETCH_PERPENDICULAR_INPUT_ERROR": "SKETCH_PERPENDICULAR_INPUT_ERROR",
            "SKETCH_PERPENDICULAR_FAILED": "SKETCH_PERPENDICULAR_FAILED",
            "SKETCH_POINT_LINE_ONLY": "SKETCH_POINT_LINE_ONLY",
            "SKETCH_PROJECTION_UNKNOWN": "SKETCH_PROJECTION_UNKNOWN",
            "SKETCH_PROJECTION_FAILED": "SKETCH_PROJECTION_FAILED",
            "SKETCH_SIL_PROJECTION_INPUT_ERROR": "SKETCH_SIL_PROJECTION_INPUT_ERROR",
            "SKETCH_SIL_PROJECTION_MISSING_POINT": "SKETCH_SIL_PROJECTION_MISSING_POINT",
            "SKETCH_LENGTH_DIM_INPUT_ERROR": "SKETCH_LENGTH_DIM_INPUT_ERROR",
            "SKETCH_LENGTH_DIM_MISSING_ENDS": "SKETCH_LENGTH_DIM_MISSING_ENDS",
            "SKETCH_LENGTH_DIM_NOT_FOUND": "SKETCH_LENGTH_DIM_NOT_FOUND",
            "SKETCH_LENGTH_DIM_FAILED": "SKETCH_LENGTH_DIM_FAILED",
            "SKETCH_RADIUS_INPUT_ERROR": "SKETCH_RADIUS_INPUT_ERROR",
            "SKETCH_RADIUS_DIM_FAILED": "SKETCH_RADIUS_DIM_FAILED",
            "SKETCH_TANGENT_INPUT_ERROR": "SKETCH_TANGENT_INPUT_ERROR",
            "SKETCH_TANGENT_FAILED": "SKETCH_TANGENT_FAILED",
            "PART_QUERY_FAILED": "PART_QUERY_FAILED",
            "PART_QUERY_MULTI": "PART_QUERY_MULTI",
            "MATECONNECTOR_QUERY_FAILED": "MATECONNECTOR_QUERY_FAILED",
            "MATECONNECTOR_QUERY_ORIGIN_FAILED": "MATECONNECTOR_QUERY_ORIGIN_FAILED",
            "MATECONNECTOR_QUERY_AXIS_FAILED": "MATECONNECTOR_QUERY_AXIS_FAILED",
            "MATECONNECTOR_QUERY_CSYS_FAILED": "MATECONNECTOR_QUERY_CSYS_FAILED",
            "ASSEMBLY_INSERT_WILL_CAUSE_CYCLES": "ASSEMBLY_INSERT_WILL_CAUSE_CYCLES",
            "SKETCH_MIRROR_NEED_VALID_MIRROR_LINE": "SKETCH_MIRROR_NEED_VALID_MIRROR_LINE",
            "SKETCH_MIRROR_NEED_ENTITIES_TO_MIRROR": "SKETCH_MIRROR_NEED_ENTITIES_TO_MIRROR",
            "SKETCH_MIRROR_CONSTRAINT_FAILED": "SKETCH_MIRROR_CONSTRAINT_FAILED",
            "SKETCH_MIRROR_FAILED": "SKETCH_MIRROR_FAILED",
            "SELF_INTERSECTING_CURVE_SELECTED": "SELF_INTERSECTING_CURVE_SELECTED",
            "SWEEP_START_NOT_ON_PROFILE": "SWEEP_START_NOT_ON_PROFILE",
            "PATTERN_DIRECTIONS_PARALLEL": "PATTERN_DIRECTIONS_PARALLEL",
            "MATE_OCCURRENCE_NOT_VALID": "MATE_OCCURRENCE_NOT_VALID",
            "MATE_WITHIN_SAME_GROUP": "MATE_WITHIN_SAME_GROUP",
            "EXPORT_ASSEMBLY_UNKNOWN_NODE_TYPE": "EXPORT_ASSEMBLY_UNKNOWN_NODE_TYPE",
            "EXPORT_ASSEMBLY_CREATE_INSTANCE_FAILED": "EXPORT_ASSEMBLY_CREATE_INSTANCE_FAILED",
            "EXPORT_PARTS_AS_XTS_NOT_A_BODY": "EXPORT_PARTS_AS_XTS_NOT_A_BODY",
            "EXPORT_PARTS_AS_XTS_FAILED_TO_WRITE_XT": "EXPORT_PARTS_AS_XTS_FAILED_TO_WRITE_XT",
            "MATECONNECTOR_OWNER_PART_NOT_RESOLVED": "MATECONNECTOR_OWNER_PART_NOT_RESOLVED",
            "WIRE_CREATION_PARTIAL_FAILURE": "WIRE_CREATION_PARTIAL_FAILURE",
            "SERVER_IS_IN_INVALID_STATE": "SERVER_IS_IN_INVALID_STATE",
            "SKETCH_EXTEND_FAILED": "SKETCH_EXTEND_FAILED",
            "FOLLOW_CYCLE_ERROR": "FOLLOW_CYCLE_ERROR",
            "SKETCH_FILLET_INVALID_RADIUS": "SKETCH_FILLET_INVALID_RADIUS",
            "SKETCH_CONSTRAINT_COINCIDENT_TWO_ENTITIES": "SKETCH_CONSTRAINT_COINCIDENT_TWO_ENTITIES",
            "SKETCH_CONSTRAINT_CONCENTRIC_TWO_ENTITIES": "SKETCH_CONSTRAINT_CONCENTRIC_TWO_ENTITIES",
            "SKETCH_CONSTRAINT_EQUAL_TWO_ENTITIES": "SKETCH_CONSTRAINT_EQUAL_TWO_ENTITIES",
            "SKETCH_CONSTRAINT_MIDPOINT_TWO_ENTITIES": "SKETCH_CONSTRAINT_MIDPOINT_TWO_ENTITIES",
            "EXTRUDE_NO_SELECTED_REGION": "EXTRUDE_NO_SELECTED_REGION",
            "EXTRUDE_NO_REGION_IN_SKETCH": "EXTRUDE_NO_REGION_IN_SKETCH",
            "DELETE_SELECT_PARTS": "DELETE_SELECT_PARTS",
            "COPY_SELECT_PARTS": "COPY_SELECT_PARTS",
            "SPLIT_NO_CHANGE": "SPLIT_NO_CHANGE",
            "MIRROR_NO_PLANE": "MIRROR_NO_PLANE",
            "MIRROR_SELECT_PARTS": "MIRROR_SELECT_PARTS",
            "PATTERN_CIRCULAR_NO_AXIS": "PATTERN_CIRCULAR_NO_AXIS",
            "PATTERN_SELECT_FACES": "PATTERN_SELECT_FACES",
            "PATTERN_SELECT_PARTS": "PATTERN_SELECT_PARTS",
            "PATTERN_LINEAR_NO_DIR": "PATTERN_LINEAR_NO_DIR",
            "SHELL_SELECT_FACES": "SHELL_SELECT_FACES",
            "DRAFT_SELECT_NEUTRAL": "DRAFT_SELECT_NEUTRAL",
            "DRAFT_SELECT_FACES": "DRAFT_SELECT_FACES",
            "CHAMFER_SELECT_EDGES": "CHAMFER_SELECT_EDGES",
            "FILLET_SELECT_EDGES": "FILLET_SELECT_EDGES",
            "EXTRUDE_SURF_NO_CURVE": "EXTRUDE_SURF_NO_CURVE",
            "EXTRUDE_SELECT_TERMINATING_BODY": "EXTRUDE_SELECT_TERMINATING_BODY",
            "EXTRUDE_SELECT_TERMINATING_SURFACE": "EXTRUDE_SELECT_TERMINATING_SURFACE",
            "DIRECT_EDIT_SELECT_ANCHOR": "DIRECT_EDIT_SELECT_ANCHOR",
            "REVOLVE_SURF_NO_CURVE": "REVOLVE_SURF_NO_CURVE",
            "REVOLVE_SELECT_FACES": "REVOLVE_SELECT_FACES",
            "REVOLVE_SELECT_AXIS": "REVOLVE_SELECT_AXIS",
            "SWEEP_SELECT_PROFILE": "SWEEP_SELECT_PROFILE",
            "SWEEP_SELECT_PATH": "SWEEP_SELECT_PATH",
            "DIRECT_EDIT_DELETE_SELECT_FACES": "DIRECT_EDIT_DELETE_SELECT_FACES",
            "DIRECT_EDIT_MODIFY_FILLET_SELECT": "DIRECT_EDIT_MODIFY_FILLET_SELECT",
            "DIRECT_EDIT_MODIFY_FACE_SELECT": "DIRECT_EDIT_MODIFY_FACE_SELECT",
            "DIRECT_EDIT_REPLACE_FACE_SELECT": "DIRECT_EDIT_REPLACE_FACE_SELECT",
            "DIRECT_EDIT_OFFSET_FACE_SELECT": "DIRECT_EDIT_OFFSET_FACE_SELECT",
            "DIRECT_EDIT_MOVE_FACE_SELECT": "DIRECT_EDIT_MOVE_FACE_SELECT",
            "SELECT_MATECONNECTOR": "SELECT_MATECONNECTOR",
            "OVERDEFINED_ASSEMBLY": "OVERDEFINED_ASSEMBLY",
            "PART_STUDIO_UPGRADE_SUCCESSFUL": "PART_STUDIO_UPGRADE_SUCCESSFUL",
            "PART_STUDIO_UPGRADE_FAILED": "PART_STUDIO_UPGRADE_FAILED",
            "PART_STUDIO_UPGRADE_NONE": "PART_STUDIO_UPGRADE_NONE",
            "MATE_GROUP_OCCURRENCES_UNRESOLVED": "MATE_GROUP_OCCURRENCES_UNRESOLVED",
            "SWEEP_SURF_NO_CURVE_PROFILE": "SWEEP_SURF_NO_CURVE_PROFILE",
            "MATE_RESET_HAD_NO_EFFECT": "MATE_RESET_HAD_NO_EFFECT",
            "MATECONNECTOR_MULTIPLE_OCCURRENCES": "MATECONNECTOR_MULTIPLE_OCCURRENCES",
            "MATECONNECTOR_OCCURRENCE_NOT_RESOLVED": "MATECONNECTOR_OCCURRENCE_NOT_RESOLVED",
            "ELEMENT_REFERENCE_CYCLE_DETECTED": "ELEMENT_REFERENCE_CYCLE_DETECTED",
            "MATE_OVERDEFINES_ASSEMBLY": "MATE_OVERDEFINES_ASSEMBLY",
            "MATE_CANNOT_RESOLVE_CONNECTORS": "MATE_CANNOT_RESOLVE_CONNECTORS",
            "SKETCH_EXCEEDS_BOUNDS": "SKETCH_EXCEEDS_BOUNDS",
            "SWEEP_SELF_INT": "SWEEP_SELF_INT",
            "SKETCH_UNSOLVABLE_CONSTRAINT": "SKETCH_UNSOLVABLE_CONSTRAINT",
            "RESTRUCTURE_INVALID_SOURCE_OR_TARGET": "RESTRUCTURE_INVALID_SOURCE_OR_TARGET",
            "CPLANE_INPUT_CURVE_POINT": "CPLANE_INPUT_CURVE_POINT",
            "TRANSFORM_OCCURRENCES_HAD_NO_EFFECT": "TRANSFORM_OCCURRENCES_HAD_NO_EFFECT",
            "HELIX_FAILED": "HELIX_FAILED",
            "HELIX_INPUT_CONE": "HELIX_INPUT_CONE",
            "RENDERER_NOT_AVAILABLE": "RENDERER_NOT_AVAILABLE",
            "RENDERER_FAILED_TO_RENDER": "RENDERER_FAILED_TO_RENDER",
            "EXPRESSION_FAILED_VALIDATION": "EXPRESSION_FAILED_VALIDATION",
            "VERSION_MISMATCH_ERROR": "VERSION_MISMATCH_ERROR",
            "EXTRUDE_UPTO_NEXT_NO_DIVISION": "EXTRUDE_UPTO_NEXT_NO_DIVISION",
            "MATE_BETWEEN_FIXED_OCCURRENCES": "MATE_BETWEEN_FIXED_OCCURRENCES",
            "THICKEN_SELECT_ENTITIES": "THICKEN_SELECT_ENTITIES",
            "THICKEN_FAILED": "THICKEN_FAILED",
            "WORKSPACE_UPGRADE_SUCCESSFUL": "WORKSPACE_UPGRADE_SUCCESSFUL",
            "WORKSPACE_UPGRADE_FAILED": "WORKSPACE_UPGRADE_FAILED",
            "WORKSPACE_UPGRADE_NONE": "WORKSPACE_UPGRADE_NONE",
            "SKETCH_CIRCULAR_PATTERN_FAILED": "SKETCH_CIRCULAR_PATTERN_FAILED",
            "DIRECT_EDIT_ALL_FILLET_FACES_SELECTED": "DIRECT_EDIT_ALL_FILLET_FACES_SELECTED",
            "DIRECT_EDIT_FAILED_TO_IDENTIFY_FILLETS": "DIRECT_EDIT_FAILED_TO_IDENTIFY_FILLETS",
            "PARASOLID_IMPORT_FAILED": "PARASOLID_IMPORT_FAILED",
            "FOLLOW_LEADER_HAS_NO_FUNCTIONALITY_ERROR": "FOLLOW_LEADER_HAS_NO_FUNCTIONALITY_ERROR",
            "MIRROR_SELECT_FACES": "MIRROR_SELECT_FACES",
            "RELATION_INVALID_RELATION": "RELATION_INVALID_RELATION",
            "RELATION_INVALID_MATE": "RELATION_INVALID_MATE",
            "GEAR_RELATION_INVALID_MATE_TYPES": "GEAR_RELATION_INVALID_MATE_TYPES",
            "SCREW_RELATION_INVALID_MATE_TYPES": "SCREW_RELATION_INVALID_MATE_TYPES",
            "RACK_RELATION_INVALID_MATE_TYPES": "RACK_RELATION_INVALID_MATE_TYPES",
            "ROLLING_RELATION_INVALID_MATE_TYPES": "ROLLING_RELATION_INVALID_MATE_TYPES",
            "LINEAR_RELATION_INVALID_MATE_TYPES": "LINEAR_RELATION_INVALID_MATE_TYPES",
            "RELATION_OVERDEFINED": "RELATION_OVERDEFINED",
            "RELATION_INCONSISTENT": "RELATION_INCONSISTENT",
            "RELATION_SAME_OCCURRENCES": "RELATION_SAME_OCCURRENCES",
            "SKETCH_SPLIT_FAILED": "SKETCH_SPLIT_FAILED",
            "SKETCH_CONSTRAINT_PIERCE_TWO_ENTITIES": "SKETCH_CONSTRAINT_PIERCE_TWO_ENTITIES",
            "SKETCH_PIERCE_FAILED": "SKETCH_PIERCE_FAILED",
            "MIRROR_FACE_FAILED": "MIRROR_FACE_FAILED",
            "MIRROR_BODY_FAILED": "MIRROR_BODY_FAILED",
            "SKETCH_CANNOT_PIERCE_WITH_PLANE": "SKETCH_CANNOT_PIERCE_WITH_PLANE",
            "WITH_SUPPORT_CODE": "WITH_SUPPORT_CODE",
            "FILLET_FAIL_SMOOTH": "FILLET_FAIL_SMOOTH",
            "FILLET_FAILED": "FILLET_FAILED",
            "CHAMFER_FAIL_SMOOTH": "CHAMFER_FAIL_SMOOTH",
            "CHAMFER_FAILED": "CHAMFER_FAILED",
            "BOOLEAN_OFFSET_NO_FACES": "BOOLEAN_OFFSET_NO_FACES",
            "MATE_OCCURRENCE_SUPPRESSED": "MATE_OCCURRENCE_SUPPRESSED",
            "MATECONNECTOR_OCCURRENCE_SUPPRESSED": "MATECONNECTOR_OCCURRENCE_SUPPRESSED",
            "SKETCH_SPLINE_NEW_POINTS_TOO_CLOSE": "SKETCH_SPLINE_NEW_POINTS_TOO_CLOSE",
            "SKETCH_SPLINE_CANNOT_DELETE_ENDPOINTS": "SKETCH_SPLINE_CANNOT_DELETE_ENDPOINTS",
            "SKETCH_SPLINE_POINT_TO_DELETE_NOT_FOUND": "SKETCH_SPLINE_POINT_TO_DELETE_NOT_FOUND",
            "ASSEMBLY_INSERT_FAILED": "ASSEMBLY_INSERT_FAILED",
            "SKETCH_PATTERN_UNKNOWN_FAILURE": "SKETCH_PATTERN_UNKNOWN_FAILURE",
            "SKETCH_PATTERN_TOO_LARGE": "SKETCH_PATTERN_TOO_LARGE",
            "SKETCH_LINEAR_PATTERN_ZERO_LENGTH": "SKETCH_LINEAR_PATTERN_ZERO_LENGTH",
            "SKETCH_LINEAR_PATTERN_PARALLEL_DIRECTIONS": "SKETCH_LINEAR_PATTERN_PARALLEL_DIRECTIONS",
            "SKETCH_CIRCULAR_PATTERN_ZERO_ANGLE": "SKETCH_CIRCULAR_PATTERN_ZERO_ANGLE",
            "SKETCH_ELLIPSE_FAILED": "SKETCH_ELLIPSE_FAILED",
            "SKETCH_ELLIPSE_FAILED_TOO_SMALL": "SKETCH_ELLIPSE_FAILED_TOO_SMALL",
            "DELETE_PARTS_FAILED": "DELETE_PARTS_FAILED",
            "DELETE_PARTS_PARTIAL": "DELETE_PARTS_PARTIAL",
            "SKETCH_ELLIPSE_RADIUS_INPUT_ERROR": "SKETCH_ELLIPSE_RADIUS_INPUT_ERROR",
            "QUADRANT_CONSTRAINT_INPUT": "QUADRANT_CONSTRAINT_INPUT",
            "SKETCH_QUADRANT_FAILED": "SKETCH_QUADRANT_FAILED",
            "SKETCH_SPLINE_TOO_FEW_POINTS": "SKETCH_SPLINE_TOO_FEW_POINTS",
            "SKETCH_SPLINE_NOT_INTERPOLATED_SPLINE": "SKETCH_SPLINE_NOT_INTERPOLATED_SPLINE",
            "SKETCH_SPLINE_POINTS_NOT_DELETED": "SKETCH_SPLINE_POINTS_NOT_DELETED",
            "SKETCH_TEXT_RECTANGLE_FAILED": "SKETCH_TEXT_RECTANGLE_FAILED",
            "IMPORT_DERIVED_NO_PARTS": "IMPORT_DERIVED_NO_PARTS",
            "LOFT_SELECT_PROFILES": "LOFT_SELECT_PROFILES",
            "LOFT_PROFILE_SINGLE_FACE": "LOFT_PROFILE_SINGLE_FACE",
            "LOFT_PROFILE_SOLID": "LOFT_PROFILE_SOLID",
            "LOFT_PROFILE_POINT": "LOFT_PROFILE_POINT",
            "LOFT_PROFILE_FAILED": "LOFT_PROFILE_FAILED",
            "LOFT_SELECT_GUIDES": "LOFT_SELECT_GUIDES",
            "LOFT_GUIDE_FAILED": "LOFT_GUIDE_FAILED",
            "LOFT_PERIODIC_ERROR": "LOFT_PERIODIC_ERROR",
            "LOFT_GUIDE_POINT_INTERSECTION": "LOFT_GUIDE_POINT_INTERSECTION",
            "LOFT_GUIDE_PROFILE_INTERSECTION": "LOFT_GUIDE_PROFILE_INTERSECTION",
            "LOFT_VERTEX_MATCHING": "LOFT_VERTEX_MATCHING",
            "LOFT_DIRECTION_ERROR": "LOFT_DIRECTION_ERROR",
            "LOFT_PROFILE_ALIGNMENT": "LOFT_PROFILE_ALIGNMENT",
            "LOFT_GUIDE_ALIGNMENT": "LOFT_GUIDE_ALIGNMENT",
            "LOFT_VERTEX_ADDITIONS": "LOFT_VERTEX_ADDITIONS",
            "LOFT_FAILED": "LOFT_FAILED",
            "LOFT_INVALID": "LOFT_INVALID",
            "LOFT_ALIGNMENT_INFO": "LOFT_ALIGNMENT_INFO",
            "LOFT_VERTEX_NOT_ON_PROFILE": "LOFT_VERTEX_NOT_ON_PROFILE",
            "LOFT_PROFILE_NO_INNER_LOOPS": "LOFT_PROFILE_NO_INNER_LOOPS",
            "LOFT_TWO_PROFILES": "LOFT_TWO_PROFILES",
            "CANNOT_OFFSET_ELLIPSE": "CANNOT_OFFSET_ELLIPSE",
            "SKETCH_MIRROR_NEEDS_LINE_AND_TWO_OTHERS": "SKETCH_MIRROR_NEEDS_LINE_AND_TWO_OTHERS",
            "SKETCH_POLYGON_BAD_SIDE_COUNT": "SKETCH_POLYGON_BAD_SIDE_COUNT",
            "SKETCH_DIRECTIONAL_GROUP_INPUT": "SKETCH_DIRECTIONAL_GROUP_INPUT",
            "NAMED_VIEWS_DUPLICATE_NAME": "NAMED_VIEWS_DUPLICATE_NAME",
            "SILHOUETTE_USE_FAILED": "SILHOUETTE_USE_FAILED",
            "PASTE_SKETCH_METRICS_FAILURE": "PASTE_SKETCH_METRICS_FAILURE",
            "PASTE_SKETCH_LIBRARY_MISMATCH": "PASTE_SKETCH_LIBRARY_MISMATCH",
            "PASTE_SKETCH_CLIPBOARD_EMPTY": "PASTE_SKETCH_CLIPBOARD_EMPTY",
            "SKETCH_MIRROR_OFFSET_SPLINE": "SKETCH_MIRROR_OFFSET_SPLINE",
            "SKETCH_MIRROR_CURVE_POINT": "SKETCH_MIRROR_CURVE_POINT",
            "LOFT_PERIODIC_GUIDE_ERROR": "LOFT_PERIODIC_GUIDE_ERROR",
            "SHELL_SELECT_PARTS": "SHELL_SELECT_PARTS",
            "RELATION_MATE_DOES_NOT_EXIST": "RELATION_MATE_DOES_NOT_EXIST",
            "RELATION_MATE_IS_SUPPRESSED": "RELATION_MATE_IS_SUPPRESSED",
            "VARIABLE_NAME_INVALID": "VARIABLE_NAME_INVALID",
            "LOFT_GUIDE_INFO": "LOFT_GUIDE_INFO",
            "HOLE_NO_POINTS": "HOLE_NO_POINTS",
            "HOLE_FAIL_BBOX": "HOLE_FAIL_BBOX",
            "HOLE_EMPTY_SCOPE": "HOLE_EMPTY_SCOPE",
            "HOLE_NO_HITS": "HOLE_NO_HITS",
            "WITH_EXTRA_DATA": "WITH_EXTRA_DATA",
            "HOLE_DISJOINT": "HOLE_DISJOINT",
            "SKETCH_INSERT_DWG_CONVERSION_FAILURE": "SKETCH_INSERT_DWG_CONVERSION_FAILURE",
            "HOLE_CBORE_TOO_SMALL": "HOLE_CBORE_TOO_SMALL",
            "HOLE_CBORE_TOO_DEEP": "HOLE_CBORE_TOO_DEEP",
            "HOLE_CSINK_TOO_SMALL": "HOLE_CSINK_TOO_SMALL",
            "HOLE_CSINK_TOO_DEEP": "HOLE_CSINK_TOO_DEEP",
            "SWEEP_PATH_NO_CONSTRUCTION": "SWEEP_PATH_NO_CONSTRUCTION",
            "SKETCH_IMAGE_RECTANGLE_FAILED": "SKETCH_IMAGE_RECTANGLE_FAILED",
            "ASSEMBLY_REPLICATE_NO_VALID_TARGET": "ASSEMBLY_REPLICATE_NO_VALID_TARGET",
            "ASSEMBLY_REPLICATE_NO_MATCHING_TARGET": "ASSEMBLY_REPLICATE_NO_MATCHING_TARGET",
            "LOFT_SHAPE_CONTROL_FAILED": "LOFT_SHAPE_CONTROL_FAILED",
            "LOFT_START_CONDITIONS_FAILED": "LOFT_START_CONDITIONS_FAILED",
            "LOFT_END_CONDITIONS_FAILED": "LOFT_END_CONDITIONS_FAILED",
            "LOFT_NO_FACE_FOR_START_CLAMP": "LOFT_NO_FACE_FOR_START_CLAMP",
            "LOFT_NO_FACE_FOR_END_CLAMP": "LOFT_NO_FACE_FOR_END_CLAMP",
            "LOFT_NO_PLANE_FOR_START_CLAMP": "LOFT_NO_PLANE_FOR_START_CLAMP",
            "LOFT_NO_PLANE_FOR_END_CLAMP": "LOFT_NO_PLANE_FOR_END_CLAMP",
            "LOFT_NO_CLAMPS_ON_POINT_PROFILE": "LOFT_NO_CLAMPS_ON_POINT_PROFILE",
            "EXPORT_NOT_IMPLEMENTED": "EXPORT_NOT_IMPLEMENTED",
            "SKETCH_POLYGON_ZERO_RADIUS_FAIL": "SKETCH_POLYGON_ZERO_RADIUS_FAIL",
            "DRAWING_FAILED_TO_RESOLVE_VIEW_REFERENCE": "DRAWING_FAILED_TO_RESOLVE_VIEW_REFERENCE",
            "DRAWING_PARTSTUDIO_EMPTY_AFTER_SECTION_CUT": "DRAWING_PARTSTUDIO_EMPTY_AFTER_SECTION_CUT",
            "DRAWING_ASSEMBLY_DOES_NOT_CONTAIN_VISIBLE_INSTANCES": "DRAWING_ASSEMBLY_DOES_NOT_CONTAIN_VISIBLE_INSTANCES",
            "DRAWING_ASSEMBLY_EMPTY_AFTER_SECTION_CUT": "DRAWING_ASSEMBLY_EMPTY_AFTER_SECTION_CUT",
            "DRAWING_VIEW_GENERATION_FAILED": "DRAWING_VIEW_GENERATION_FAILED",
            "SKETCH_SLOT_FAILURE": "SKETCH_SLOT_FAILURE",
            "SKETCH_SLOT_PARTIAL_FAILURE": "SKETCH_SLOT_PARTIAL_FAILURE",
            "NO_UNIT": "NO_UNIT",
            "RESTRUCTURE_INVALID_SOURCE": "RESTRUCTURE_INVALID_SOURCE",
            "RESTRUCTURE_INVALID_TARGET": "RESTRUCTURE_INVALID_TARGET",
            "MATE_MIN_MAX_LIMIT_VIOLATION": "MATE_MIN_MAX_LIMIT_VIOLATION",
            "REST_ASSEMBLY_GET_DOCUMENT_FAILED": "REST_ASSEMBLY_GET_DOCUMENT_FAILED",
            "REST_ASSEMBLY_UNKNOWN_INSERTABLE_TYPE": "REST_ASSEMBLY_UNKNOWN_INSERTABLE_TYPE",
            "REST_ASSEMBLY_SETUP_EXCEPTION": "REST_ASSEMBLY_SETUP_EXCEPTION",
            "REST_ASSEMBLY_BEGIN_OPERATION_FAILED": "REST_ASSEMBLY_BEGIN_OPERATION_FAILED",
            "REST_ASSEMBLY_INSERT_INSTANCE_FAILED": "REST_ASSEMBLY_INSERT_INSTANCE_FAILED",
            "REST_ASSEMBLY_COMMIT_OPERATION_FAILED": "REST_ASSEMBLY_COMMIT_OPERATION_FAILED",
            "REST_ASSEMBLY_CLOSE_CLIENT_FAILED": "REST_ASSEMBLY_CLOSE_CLIENT_FAILED",
            "REST_ASSEMBLY_NULL_OCCURRENCES": "REST_ASSEMBLY_NULL_OCCURRENCES",
            "REST_ASSEMBLY_EMPTY_OCCURRENCE": "REST_ASSEMBLY_EMPTY_OCCURRENCE",
            "REST_ASSEMBLY_TRANSFORM_WRONG_SIZE": "REST_ASSEMBLY_TRANSFORM_WRONG_SIZE",
            "ASSEMBLY_EMPTY_OCCURRENCE_LIST": "ASSEMBLY_EMPTY_OCCURRENCE_LIST",
            "ASSEMBLY_NULL_TRANSFORM": "ASSEMBLY_NULL_TRANSFORM",
            "ASSEMBLY_TRANSFORM_NOT_RIGID": "ASSEMBLY_TRANSFORM_NOT_RIGID",
            "ASSEMBLY_CANNOT_TRANSFORM_FIXED_OCCURRENCE": "ASSEMBLY_CANNOT_TRANSFORM_FIXED_OCCURRENCE",
            "ASSEMBLY_TRANSFORM_FAILED": "ASSEMBLY_TRANSFORM_FAILED",
            "ASSEMBLY_OCCURRENCE_NOT_FOUND": "ASSEMBLY_OCCURRENCE_NOT_FOUND",
            "ASSEMBLY_REPLICATE_MULTIPLE_VALID_TARGET": "ASSEMBLY_REPLICATE_MULTIPLE_VALID_TARGET",
            "ASSEMBLY_REPLICATE_NO_TARGET_SELECTED": "ASSEMBLY_REPLICATE_NO_TARGET_SELECTED",
            "CPLANE_INPUT_LINE_ANGLE2": "CPLANE_INPUT_LINE_ANGLE2",
            "CPLANE_DEGENERATE_SELECTION": "CPLANE_DEGENERATE_SELECTION",
            "CPLANE_SELECT_LINE_ANGLE_REFERENCE": "CPLANE_SELECT_LINE_ANGLE_REFERENCE",
            "ASSEMBLY_REPLICATE_INVALID_SEED_INSTANCE": "ASSEMBLY_REPLICATE_INVALID_SEED_INSTANCE",
            "CANNOT_USE_VARIABLES_IN_SKETCH_PATTERNS": "CANNOT_USE_VARIABLES_IN_SKETCH_PATTERNS",
            "SKETCH_MIRROR_OFFSET_ELLIPSE": "SKETCH_MIRROR_OFFSET_ELLIPSE",
            "EXTERNAL_REFERENCE_FAILED_TO_CREATE": "EXTERNAL_REFERENCE_FAILED_TO_CREATE",
            "SPLIT_FACE_NO_CHANGE": "SPLIT_FACE_NO_CHANGE",
            "SKETCH_INTERSECTION_FAILED": "SKETCH_INTERSECTION_FAILED",
            "SKETCH_INTERSECTION_MULTIPLE_FAILED": "SKETCH_INTERSECTION_MULTIPLE_FAILED",
            "SKETCH_INTERSECTION_PARTIAL_FAILED": "SKETCH_INTERSECTION_PARTIAL_FAILED",
            "FEATURE_ID_IN_PATH_DOES_NOT_MATCH_BODY": "FEATURE_ID_IN_PATH_DOES_NOT_MATCH_BODY",
            "FEATURE_NOT_FOUND": "FEATURE_NOT_FOUND",
            "FEATURE_DOES_NOT_MATCH": "FEATURE_DOES_NOT_MATCH",
            "FEATURE_HAS_INVALID_TYPE": "FEATURE_HAS_INVALID_TYPE",
            "FEATURE_DOES_NOT_MATCH_ITS_FEATURE_SPEC": "FEATURE_DOES_NOT_MATCH_ITS_FEATURE_SPEC",
            "FEATURE_BAD_SERIALIZATION_VERSION": "FEATURE_BAD_SERIALIZATION_VERSION",
            "FEATURE_WRONG_SERIALIZATION_VERSION": "FEATURE_WRONG_SERIALIZATION_VERSION",
            "FEATURE_INVALID_ROLLBACK_INDEX": "FEATURE_INVALID_ROLLBACK_INDEX",
            "FEATURE_ERROR_IN_INPUT": "FEATURE_ERROR_IN_INPUT",
            "FEATURE_CONCURRENCY_ERROR": "FEATURE_CONCURRENCY_ERROR",
            "FEATURE_CHANGE_BREAKS_MODEL": "FEATURE_CHANGE_BREAKS_MODEL",
            "FEATURE_NODE_IDS_INVALID": "FEATURE_NODE_IDS_INVALID",
            "ROLLBACK_INDEX_INVALID": "ROLLBACK_INDEX_INVALID",
            "FEATURE_NO_SOLIDS": "FEATURE_NO_SOLIDS",
            "SKETCH_EXTERNAL_GEOMETRY_MISMATCH": "SKETCH_EXTERNAL_GEOMETRY_MISMATCH",
            "HOLE_EXCEEDS_MAX_LOCATIONS": "HOLE_EXCEEDS_MAX_LOCATIONS",
            "SKETCH_TEXT_IS_EMPTY": "SKETCH_TEXT_IS_EMPTY",
            "SKETCH_INSERT_DWG_MAX_ENTITIES_EXCEEDED": "SKETCH_INSERT_DWG_MAX_ENTITIES_EXCEEDED",
            "HOLE_TAP_DIA_TOO_LARGE": "HOLE_TAP_DIA_TOO_LARGE",
            "ASSEMBLY_EMPTY_BODY": "ASSEMBLY_EMPTY_BODY",
            "SIMPLIFY_BODY_FAILED": "SIMPLIFY_BODY_FAILED",
            "INVALID_VIEW_NAME": "INVALID_VIEW_NAME",
            "PATTERN_SELECT_FEATURES": "PATTERN_SELECT_FEATURES",
            "MIRROR_SELECT_FEATURES": "MIRROR_SELECT_FEATURES",
            "PATTERN_FEATURE_FAILED": "PATTERN_FEATURE_FAILED",
            "SKETCH_TRANSFORM_FAILED": "SKETCH_TRANSFORM_FAILED",
            "TANGENT_MATE_TWO_ENTITIES_NEEDED": "TANGENT_MATE_TWO_ENTITIES_NEEDED",
            "HOLE_CANNOT_DETERMINE_LAST_BODY": "HOLE_CANNOT_DETERMINE_LAST_BODY",
            "RESTRUCTURE_CANNOT_MODIFY_SAVED_VERSION": "RESTRUCTURE_CANNOT_MODIFY_SAVED_VERSION",
            "REST_ASSEMBLY_EXTERNAL_REFERENCE_REQUIRES_VERSION": "REST_ASSEMBLY_EXTERNAL_REFERENCE_REQUIRES_VERSION",
            "REST_ASSEMBLY_EXTERNAL_REFERENCE_DISALLOWS_MICROVERSION": "REST_ASSEMBLY_EXTERNAL_REFERENCE_DISALLOWS_MICROVERSION",
            "REST_ASSEMBLY_VERSION_SUPPORTED_ONLY_FOR_EXTERNAL_REFERENCES": "REST_ASSEMBLY_VERSION_SUPPORTED_ONLY_FOR_EXTERNAL_REFERENCES",
            "SWEEP_BAD_LOCK_FACES": "SWEEP_BAD_LOCK_FACES",
            "SKETCH_TEXT_CANNOT_BE_CONSTRUCTION": "SKETCH_TEXT_CANNOT_BE_CONSTRUCTION",
            "CUSTOM_ERROR": "CUSTOM_ERROR",
            "BEND_BAD_CONFIGURATION": "BEND_BAD_CONFIGURATION",
            "BEND_WRONG_NUMBER_OF_ENTITIES": "BEND_WRONG_NUMBER_OF_ENTITIES",
            "BEND_BAD_CURVES": "BEND_BAD_CURVES",
            "BEND_GENERAL_ERROR": "BEND_GENERAL_ERROR",
            "BEND_EDGE_NO_EDGES": "BEND_EDGE_NO_EDGES",
            "BEND_EDGE_NO_SEED_ENTITY": "BEND_EDGE_NO_SEED_ENTITY",
            "EXTEND_SHEET_BODY_NO_BODY": "EXTEND_SHEET_BODY_NO_BODY",
            "EXTRACT_SURFACE_NO_FACES": "EXTRACT_SURFACE_NO_FACES",
            "FLATTEN_NO_EDGES": "FLATTEN_NO_EDGES",
            "FLATTEN_NO_FACES": "FLATTEN_NO_FACES",
            "FOLD_NO_BODIES": "FOLD_NO_BODIES",
            "BEND_PREP_NO_FACES": "BEND_PREP_NO_FACES",
            "BEND_PREP_NO_BODIES": "BEND_PREP_NO_BODIES",
            "BEND_PREP_ERROR_FINDING_EDGE_LOCATIONS": "BEND_PREP_ERROR_FINDING_EDGE_LOCATIONS",
            "BEND_PREP_ERROR_IMPRINTING_EDGES": "BEND_PREP_ERROR_IMPRINTING_EDGES",
            "ASSEMBLY_ANIMATE_MATE_START_AFTER_END": "ASSEMBLY_ANIMATE_MATE_START_AFTER_END",
            "ASSEMBLY_ANIMATE_NO_MATE": "ASSEMBLY_ANIMATE_NO_MATE",
            "ASSEMBLY_ANIMATE_MATE_SUPPRESSED": "ASSEMBLY_ANIMATE_MATE_SUPPRESSED",
            "TANGENT_MATE_GEOMETRY_NOT_SUPPORTED": "TANGENT_MATE_GEOMETRY_NOT_SUPPORTED",
            "SKETCH_DIMENSION_INFINITY": "SKETCH_DIMENSION_INFINITY",
            "BOLEAN_INPUTS_NOT_SOLID": "BOLEAN_INPUTS_NOT_SOLID",
            "FACE_IS_NOT_RECTANGLE": "FACE_IS_NOT_RECTANGLE",
            "HOLE_DESTROY_SOLID": "HOLE_DESTROY_SOLID",
            "HELIX_INPUT_CIRCLE": "HELIX_INPUT_CIRCLE",
            "IMPORT_SCALING_NON_MESH_DATA": "IMPORT_SCALING_NON_MESH_DATA",
            "EVALUATE_FACE_TANGENT_FOR_MESHES": "EVALUATE_FACE_TANGENT_FOR_MESHES",
            "CANNOT_COMPUTE_CENTROID": "CANNOT_COMPUTE_CENTROID",
            "CANNOT_EVALUATE_DIMENSION": "CANNOT_EVALUATE_DIMENSION",
            "CANNOT_IMPORT_MESH": "CANNOT_IMPORT_MESH",
            "SKETCH_ELLIPSE_ZERO_AXIS": "SKETCH_ELLIPSE_ZERO_AXIS",
            "TRANSFORM_SCALE_UNIFORMLY": "TRANSFORM_SCALE_UNIFORMLY",
            "TRANSFORM_MATE_CONNECTORS": "TRANSFORM_MATE_CONNECTORS",
            "ASSEMBLY_WRONG_ELEMENT_TYPE": "ASSEMBLY_WRONG_ELEMENT_TYPE",
            "ASSEMBLY_ELEMENT_NOT_FOUND": "ASSEMBLY_ELEMENT_NOT_FOUND",
            "SHEET_METAL_TABLE_UNKNOWN_ERROR": "SHEET_METAL_TABLE_UNKNOWN_ERROR",
            "SHEET_METAL_TABLE_REGEN_ERROR": "SHEET_METAL_TABLE_REGEN_ERROR",
            "SHEET_METAL_TABLE_READ_ONLY": "SHEET_METAL_TABLE_READ_ONLY",
            "ASSEMBLY_PATTERN_INVALID_TYPE": "ASSEMBLY_PATTERN_INVALID_TYPE",
            "ASSEMBLY_PATTERN_DIRECTION_ERROR": "ASSEMBLY_PATTERN_DIRECTION_ERROR",
            "ASSEMBLY_PATTERN_NONPOSITIVE_LINEAR_DISTANCE": "ASSEMBLY_PATTERN_NONPOSITIVE_LINEAR_DISTANCE",
            "ASSEMBLY_PATTERN_NONPOSITIVE_ANGLE": "ASSEMBLY_PATTERN_NONPOSITIVE_ANGLE",
            "ASSEMBLY_PATTERN_INVALID_SEED": "ASSEMBLY_PATTERN_INVALID_SEED",
            "ASSEMBLY_PATTERN_INVALID_REFERENCE_MATE_CONNECTOR": "ASSEMBLY_PATTERN_INVALID_REFERENCE_MATE_CONNECTOR",
            "RESTORE_FEATURE_FAILED": "RESTORE_FEATURE_FAILED",
            "FACES_NOT_OWNED_BY_PARTS": "FACES_NOT_OWNED_BY_PARTS",
            "EDGES_NOT_OWNED_BY_PARTS": "EDGES_NOT_OWNED_BY_PARTS",
            "SHEET_METAL_REBUILD_ERROR": "SHEET_METAL_REBUILD_ERROR",
            "SHEET_METAL_INPUT_BODY_SHOULD_NOT_BE_SHEET_METAL": "SHEET_METAL_INPUT_BODY_SHOULD_NOT_BE_SHEET_METAL",
            "SHEET_METAL_CANNOT_RECOGNIZE_PARTS": "SHEET_METAL_CANNOT_RECOGNIZE_PARTS",
            "SHEET_METAL_CANNOT_THICKEN": "SHEET_METAL_CANNOT_THICKEN",
            "SHEET_METAL_CONVERT_PLANE": "SHEET_METAL_CONVERT_PLANE",
            "ASSEMBLY_PATTERN_AXIS_ERROR": "ASSEMBLY_PATTERN_AXIS_ERROR",
            "RIB_NO_PROFILES": "RIB_NO_PROFILES",
            "RIB_NO_PARTS": "RIB_NO_PARTS",
            "RIB_PROFILE_FAILED": "RIB_PROFILE_FAILED",
            "RIB_BODY_FAILED": "RIB_BODY_FAILED",
            "RIB_NO_INTERSECTIONS": "RIB_NO_INTERSECTIONS",
            "RIB_MERGE_FAILED": "RIB_MERGE_FAILED",
            "ASSEMBLY_NAMED_POSITIONS_SAVE_FAILED": "ASSEMBLY_NAMED_POSITIONS_SAVE_FAILED",
            "ASSEMBLY_NAMED_POSITIONS_LOAD_FAILED": "ASSEMBLY_NAMED_POSITIONS_LOAD_FAILED",
            "ASSEMBLY_NAMED_POSITIONS_NO_MATES_TO_SAVE": "ASSEMBLY_NAMED_POSITIONS_NO_MATES_TO_SAVE",
            "ASSEMBLY_NAMED_POSITIONS_POSITION_NOT_FOUND": "ASSEMBLY_NAMED_POSITIONS_POSITION_NOT_FOUND",
            "SPHERE_FAILED": "SPHERE_FAILED",
            "ASSEMBLY_PATTERN_NOT_SUPPORTED": "ASSEMBLY_PATTERN_NOT_SUPPORTED",
            "ASSEMBLY_NAMED_POSITIONS_PARTIAL_LOAD_FAILURE": "ASSEMBLY_NAMED_POSITIONS_PARTIAL_LOAD_FAILURE",
            "ASSEMBLY_NAMED_POSITIONS_LOAD_SUCCEEDED_WITH_EXTRA_MATES": "ASSEMBLY_NAMED_POSITIONS_LOAD_SUCCEEDED_WITH_EXTRA_MATES",
            "ASSEMBLY_NAMED_POSITIONS_SAVED_MATE_NOT_FOUND_ON_LOAD": "ASSEMBLY_NAMED_POSITIONS_SAVED_MATE_NOT_FOUND_ON_LOAD",
            "SHEET_METAL_SINGLE_MODEL_NEEDED": "SHEET_METAL_SINGLE_MODEL_NEEDED",
            "SHEET_METAL_ACTIVE_JOIN_NEEDED": "SHEET_METAL_ACTIVE_JOIN_NEEDED",
            "INSTANCE_QUERY_FAILED": "INSTANCE_QUERY_FAILED",
            "SHEET_METAL_ACTIVE_EDGE_NEEDED": "SHEET_METAL_ACTIVE_EDGE_NEEDED",
            "SHEET_METAL_FLANGE_NO_EDGES": "SHEET_METAL_FLANGE_NO_EDGES",
            "MESH_NOT_SUPPORTED": "MESH_NOT_SUPPORTED",
            "SHEET_METAL_PARTS_PROHIBITED": "SHEET_METAL_PARTS_PROHIBITED",
            "VARIABLE_CANNOT_EVALUATE": "VARIABLE_CANNOT_EVALUATE",
            "DRAWING_ASSEMBLY_INVALID_SECTION_CUT": "DRAWING_ASSEMBLY_INVALID_SECTION_CUT",
            "DRAWING_PARTSTUDIO_INVALID_SECTION_CUT": "DRAWING_PARTSTUDIO_INVALID_SECTION_CUT",
            "SHEET_METAL_COULD_NOT_UNFOLD": "SHEET_METAL_COULD_NOT_UNFOLD",
            "PARAMETER_OUT_OF_RANGE": "PARAMETER_OUT_OF_RANGE",
            "SHEET_METAL_NO_0_ANGLE_BEND": "SHEET_METAL_NO_0_ANGLE_BEND",
            "SHEET_METAL_FLAT_RIP_NO_EDIT": "SHEET_METAL_FLAT_RIP_NO_EDIT",
            "SHEET_METAL_CANT_CHANGE_TO_FLAT": "SHEET_METAL_CANT_CHANGE_TO_FLAT",
            "PARAMETER_PRECONDITION_FAILED": "PARAMETER_PRECONDITION_FAILED",
            "PARAMETER_SYNTAX_ERROR": "PARAMETER_SYNTAX_ERROR",
            "SHEET_METAL_CAN_ONLY_REMOVE": "SHEET_METAL_CAN_ONLY_REMOVE",
            "SHEET_METAL_CAN_ONLY_SUBTRACT": "SHEET_METAL_CAN_ONLY_SUBTRACT",
            "REST_ASSEMBLY_INVALID_FEATURE": "REST_ASSEMBLY_INVALID_FEATURE",
            "REST_ASSEMBLY_INVALID_BODY_TYPE": "REST_ASSEMBLY_INVALID_BODY_TYPE",
            "PARTING_OUT_TARGET_READONLY": "PARTING_OUT_TARGET_READONLY",
            "SHEET_METAL_MULTI_SM_DEFAULT_RADIUS": "SHEET_METAL_MULTI_SM_DEFAULT_RADIUS",
            "SHEET_METAL_FLANGE_FAIL_ALIGNMENT": "SHEET_METAL_FLANGE_FAIL_ALIGNMENT",
            "SHEET_METAL_FLANGE_FAIL_UP_TO": "SHEET_METAL_FLANGE_FAIL_UP_TO",
            "SHEET_METAL_FLANGE_FAIL_UP_TO_ENTITY": "SHEET_METAL_FLANGE_FAIL_UP_TO_ENTITY",
            "SHEET_METAL_FLANGE_FAIL": "SHEET_METAL_FLANGE_FAIL",
            "SHEET_METAL_FLANGE_FAIL_LIMIT_OPP_FLANGE": "SHEET_METAL_FLANGE_FAIL_LIMIT_OPP_FLANGE",
            "CANT_SPLIT_SHEET_METAL_BEND_FACE": "CANT_SPLIT_SHEET_METAL_BEND_FACE",
            "IN_CONTEXT_INSTANCE_INVALID_TARGET": "IN_CONTEXT_INSTANCE_INVALID_TARGET",
            "SHEET_METAL_SELF_INTERSECTING_MODEL": "SHEET_METAL_SELF_INTERSECTING_MODEL",
            "SHEET_METAL_SELF_INTERSECTING_FLAT": "SHEET_METAL_SELF_INTERSECTING_FLAT",
            "SHEET_METAL_NON_90_BUTT": "SHEET_METAL_NON_90_BUTT",
            "SHEET_METAL_RIP_STYLE_ERROR": "SHEET_METAL_RIP_STYLE_ERROR",
            "CANNOT_USE_MATECONNECTORS_IN_PATTERN": "CANNOT_USE_MATECONNECTORS_IN_PATTERN",
            "CANNOT_COPY_MATECONNECTORS": "CANNOT_COPY_MATECONNECTORS",
            "SHEET_METAL_CAN_ONLY_OFFSET": "SHEET_METAL_CAN_ONLY_OFFSET",
            "MODIFIABLE_ENTITY_ONLY": "MODIFIABLE_ENTITY_ONLY",
            "IN_CONTEXT_UPDATE_DELETED_ASSEMBLY": "IN_CONTEXT_UPDATE_DELETED_ASSEMBLY",
            "IN_CONTEXT_UPDATE_EMPTY_INSTANCE": "IN_CONTEXT_UPDATE_EMPTY_INSTANCE",
            "IN_CONTEXT_UPDATE_INVALID_SOURCE": "IN_CONTEXT_UPDATE_INVALID_SOURCE",
            "IN_CONTEXT_UPDATE_INVALID_TARGET": "IN_CONTEXT_UPDATE_INVALID_TARGET",
            "SHEET_METAL_NO_FEATURE_PATTERN": "SHEET_METAL_NO_FEATURE_PATTERN",
            "CUSTOM_FEATURE_DEFINITION_NOT_FOUND": "CUSTOM_FEATURE_DEFINITION_NOT_FOUND",
            "SHEET_METAL_START_SELECT_BENDS": "SHEET_METAL_START_SELECT_BENDS",
            "SHEET_METAL_END_DONE": "SHEET_METAL_END_DONE",
            "PATH_EDGES_NOT_CONTINUOUS": "PATH_EDGES_NOT_CONTINUOUS",
            "SHEET_METAL_RIP_FAIL_INTERNAL_EDGE": "SHEET_METAL_RIP_FAIL_INTERNAL_EDGE",
            "SHEET_METAL_RIP_FAIL": "SHEET_METAL_RIP_FAIL",
            "SHEET_METAL_RIP_MULTI_BODY": "SHEET_METAL_RIP_MULTI_BODY",
            "SHEET_METAL_RIP_FAIL_NON_PLANAR": "SHEET_METAL_RIP_FAIL_NON_PLANAR",
            "PATTERN_CURVE_NO_EDGES": "PATTERN_CURVE_NO_EDGES",
            "SHEET_METAL_RIP_NO_CORNER": "SHEET_METAL_RIP_NO_CORNER",
            "SHEET_METAL_RIP_EVEN": "SHEET_METAL_RIP_EVEN",
            "SHEET_METAL_RIP_WALL_NOT_FOUND": "SHEET_METAL_RIP_WALL_NOT_FOUND",
            "SHEET_METAL_RIP_SAME_VERTEX": "SHEET_METAL_RIP_SAME_VERTEX",
            "SHEET_METAL_RIP_NEED_MORE_VERTICES": "SHEET_METAL_RIP_NEED_MORE_VERTICES",
            "SHEET_METAL_MAKE_JOINT_FAIL": "SHEET_METAL_MAKE_JOINT_FAIL",
            "CURVE_PATTERN_START_OFF_PATH": "CURVE_PATTERN_START_OFF_PATH",
            "PART_LOAD_FAILED": "PART_LOAD_FAILED",
            "SHEET_METAL_MOVE_NOT_PLANAR": "SHEET_METAL_MOVE_NOT_PLANAR",
            "SHEET_METAL_JOINT_FAIL_ADJACENT_FACES": "SHEET_METAL_JOINT_FAIL_ADJACENT_FACES",
            "WRONG_PARASOLID_VERSION": "WRONG_PARASOLID_VERSION",
            "SHEET_METAL_SINGLE_MODEL_NEEDED_EDGES": "SHEET_METAL_SINGLE_MODEL_NEEDED_EDGES",
            "REST_ASSEMBLY_MISSING_INSTANCE_DOCUMENT_ID": "REST_ASSEMBLY_MISSING_INSTANCE_DOCUMENT_ID",
            "REST_ASSEMBLY_MISSING_INSTANCE_ELEMENT_ID": "REST_ASSEMBLY_MISSING_INSTANCE_ELEMENT_ID",
            "FACE_CLASH": "FACE_CLASH",
            "CURVE_PATTERN_START_OFF_CLOSED_PATH": "CURVE_PATTERN_START_OFF_CLOSED_PATH",
            "SHEET_METAL_CUT_JOINT": "SHEET_METAL_CUT_JOINT",
            "EXPORT_STL_NO_PARTS": "EXPORT_STL_NO_PARTS",
            "INPUT_NAME_TOO_LONG": "INPUT_NAME_TOO_LONG",
            "IMPORT_BODY_FAILED_CHECK": "IMPORT_BODY_FAILED_CHECK",
            "DERIVED_BODIES_HAVE_FAULTS": "DERIVED_BODIES_HAVE_FAULTS",
            "SHEET_METAL_BLOCKED_PATTERN": "SHEET_METAL_BLOCKED_PATTERN",
            "SHEET_METAL_FLANGE_INTERNAL": "SHEET_METAL_FLANGE_INTERNAL",
            "SHEET_METAL_TOO_THICK": "SHEET_METAL_TOO_THICK",
            "SHEET_METAL_BEND_END_NOT_A_CORNER": "SHEET_METAL_BEND_END_NOT_A_CORNER",
            "SHEET_METAL_NOT_A_CLOSED_CORNER": "SHEET_METAL_NOT_A_CLOSED_CORNER",
            "SHEET_METAL_CORNER_NOT_A_BEND_END": "SHEET_METAL_CORNER_NOT_A_BEND_END",
            "RIB_ONLY_OPEN_PROFILES": "RIB_ONLY_OPEN_PROFILES",
            "TAB_NO_LONGER_EXISTS": "TAB_NO_LONGER_EXISTS",
            "CLINE_FAILED": "CLINE_FAILED",
            "ILLEGAL_MODIFICATION": "ILLEGAL_MODIFICATION",
            "ASSEMBLY_MATE_VALUE_SET_FAILED": "ASSEMBLY_MATE_VALUE_SET_FAILED",
            "EXTRUDE_OFFSET_TOO_DEEP": "EXTRUDE_OFFSET_TOO_DEEP",
            "SHEET_METAL_CANNOT_MOVE_BEND_EDGE": "SHEET_METAL_CANNOT_MOVE_BEND_EDGE",
            "UP_TO_FACE_NOT_PARALLEL": "UP_TO_FACE_NOT_PARALLEL",
            "TRANSLATION_FACE_NOT_PLANAR": "TRANSLATION_FACE_NOT_PLANAR",
            "MOVE_FACE_NO_INTERSECTION": "MOVE_FACE_NO_INTERSECTION",
            "SWEEP_PATH_PROFILE_NO_INTERSECTION": "SWEEP_PATH_PROFILE_NO_INTERSECTION",
            "DIRECT_EDIT_MOVE_FACE_CREATE_SELECT": "DIRECT_EDIT_MOVE_FACE_CREATE_SELECT",
            "SHEET_METAL_THICKEN_IN_CONTEXT_INFO": "SHEET_METAL_THICKEN_IN_CONTEXT_INFO",
            "CANNOT_DELETE_RHO_DIMENSION": "CANNOT_DELETE_RHO_DIMENSION",
            "INVALID_RHO": "INVALID_RHO",
            "SKETCH_RHO_DIM_NOT_FOUND": "SKETCH_RHO_DIM_NOT_FOUND",
            "SKETCH_CONIC_FAILED": "SKETCH_CONIC_FAILED",
            "TAB_NAME_TOO_LONG": "TAB_NAME_TOO_LONG",
            "DIRECT_EDIT_DELETE_FACE_ALL_FACES": "DIRECT_EDIT_DELETE_FACE_ALL_FACES",
            "SHEET_METAL_CORNER_BREAK_FAILED": "SHEET_METAL_CORNER_BREAK_FAILED",
            "BOOLEAN_NO_TARGET_SURFACE": "BOOLEAN_NO_TARGET_SURFACE",
            "BAD_BSPLINECURVE_DEFINITION": "BAD_BSPLINECURVE_DEFINITION",
            "REQUIRE_3D_BSPLINECURVE_DATA": "REQUIRE_3D_BSPLINECURVE_DATA",
            "PERIODIC_BSPLINECURVE_NOT_CLOSED": "PERIODIC_BSPLINECURVE_NOT_CLOSED",
            "PERIODIC_BSPLINECURVE_NOT_SMOOTH": "PERIODIC_BSPLINECURVE_NOT_SMOOTH",
            "RATIONAL_BSPLINECURVE_WEIGHT_NEGATIVE": "RATIONAL_BSPLINECURVE_WEIGHT_NEGATIVE",
            "BSPLINECURVE_NOT_G1": "BSPLINECURVE_NOT_G1",
            "BRIDGING_CURVE_VERTEX_BOTH_SIDES": "BRIDGING_CURVE_VERTEX_BOTH_SIDES",
            "BRIDGING_CURVE_ONE_EDGE_EACH_SIDE": "BRIDGING_CURVE_ONE_EDGE_EACH_SIDE",
            "BRIDGING_CURVE_VERTEX_AT_END_OF_EDGE": "BRIDGING_CURVE_VERTEX_AT_END_OF_EDGE",
            "PROJECT_CURVES_PARALLEL_PLANES": "PROJECT_CURVES_PARALLEL_PLANES",
            "PROJECT_CURVES_DIFFERENT_SKETCHES": "PROJECT_CURVES_DIFFERENT_SKETCHES",
            "SHEET_METAL_SELECT_PART": "SHEET_METAL_SELECT_PART",
            "VARIABLE_NOT_FOUND": "VARIABLE_NOT_FOUND",
            "CANNOT_EDIT_FIXED_CONIC": "CANNOT_EDIT_FIXED_CONIC",
            "EXTRACT_WIRES_OVERLAPPING_EDGES": "EXTRACT_WIRES_OVERLAPPING_EDGES",
            "EXTRACT_WIRES_NON_MANIFOLD": "EXTRACT_WIRES_NON_MANIFOLD",
            "EXTRACT_WIRES_NEEDS_EDGES": "EXTRACT_WIRES_NEEDS_EDGES",
            "SPLINE_TWO_POINTS": "SPLINE_TWO_POINTS",
            "CLOSED_SPLINE_THREE_POINTS": "CLOSED_SPLINE_THREE_POINTS",
            "TANGENCY_ONE_EDGE": "TANGENCY_ONE_EDGE",
            "FIT_SPLINE_CANNOT_EVALUATE_END_CONDITION": "FIT_SPLINE_CANNOT_EVALUATE_END_CONDITION",
            "FIT_SPLINE_REPEATED_POINT": "FIT_SPLINE_REPEATED_POINT",
            "FEATURE_ID_REQUIRED": "FEATURE_ID_REQUIRED",
            "PARAMETER_NOT_FOUND": "PARAMETER_NOT_FOUND",
            "PARAMETER_DOES_NOT_MATCH_ITS_FEATURE_SPEC": "PARAMETER_DOES_NOT_MATCH_ITS_FEATURE_SPEC",
            "SHEET_METAL_CORNER_BREAK_DISABLED": "SHEET_METAL_CORNER_BREAK_DISABLED",
            "SHEET_METAL_FILLET_NO_CONIC": "SHEET_METAL_FILLET_NO_CONIC",
            "SHEET_METAL_CHAMFER_NO_TWO_OFFSETS": "SHEET_METAL_CHAMFER_NO_TWO_OFFSETS",
            "SHEET_METAL_CHAMFER_NO_OFFSET_ANGLE": "SHEET_METAL_CHAMFER_NO_OFFSET_ANGLE",
            "SHEET_METAL_CHAMFER_MUST_BE_EQUAL_OFFSETS": "SHEET_METAL_CHAMFER_MUST_BE_EQUAL_OFFSETS",
            "SHEET_METAL_ACTIVE_ENTITY_NEEDED": "SHEET_METAL_ACTIVE_ENTITY_NEEDED",
            "SHEET_METAL_CORNER_BREAK_NOT_A_CORNER": "SHEET_METAL_CORNER_BREAK_NOT_A_CORNER",
            "SHEET_METAL_CORNER_BREAK_NO_WALL": "SHEET_METAL_CORNER_BREAK_NO_WALL",
            "SHEET_METAL_CORNER_BREAK_VERTEX_NOT_FREE": "SHEET_METAL_CORNER_BREAK_VERTEX_NOT_FREE",
            "SHEET_METAL_CORNER_BREAK_ATTRIBUTE_EXISTS": "SHEET_METAL_CORNER_BREAK_ATTRIBUTE_EXISTS",
            "FIT_SPLINE_ZERO_START_MAGNITUDE": "FIT_SPLINE_ZERO_START_MAGNITUDE",
            "FIT_SPLINE_ZERO_END_MAGNITUDE": "FIT_SPLINE_ZERO_END_MAGNITUDE",
            "SHEET_METAL_CORNER_BREAK_SELECT_ENTITIES": "SHEET_METAL_CORNER_BREAK_SELECT_ENTITIES",
            "EXTRUDE_SELECT_TERMINATING_VERTEX": "EXTRUDE_SELECT_TERMINATING_VERTEX",
            "FILL_SURFACE_NO_EDGES": "FILL_SURFACE_NO_EDGES",
            "FILL_SURFACE_DOUBLE_SELECTION": "FILL_SURFACE_DOUBLE_SELECTION",
            "FILL_SURFACE_MULTI_LOOP": "FILL_SURFACE_MULTI_LOOP",
            "FILL_SURFACE_OPEN_LOOP": "FILL_SURFACE_OPEN_LOOP",
            "FILL_SURFACE_FAIL": "FILL_SURFACE_FAIL",
            "FILL_SURFACE_ATTACH_FAIL": "FILL_SURFACE_ATTACH_FAIL",
            "FILL_SURFACE_G2_FAIL": "FILL_SURFACE_G2_FAIL",
            "FILL_SURFACE_VERTEX_INTERPOLATION_FAIL": "FILL_SURFACE_VERTEX_INTERPOLATION_FAIL",
            "LOFT_SPINE_DISJOINT_PATH": "LOFT_SPINE_DISJOINT_PATH",
            "LOFT_SPINE_SELF_INTERSECTING_PATH": "LOFT_SPINE_SELF_INTERSECTING_PATH",
            "LOFT_SPINE_PATH_PROFILE_NO_INTERSECTION": "LOFT_SPINE_PATH_PROFILE_NO_INTERSECTION",
            "LOFT_SPINE_FAILED_XSECTIONS": "LOFT_SPINE_FAILED_XSECTIONS",
            "LOFT_SPINE_GUIDE_WITH_POINT_PROFILE": "LOFT_SPINE_GUIDE_WITH_POINT_PROFILE",
            "LOFT_SPINE_PROFILES_NOT_IN_ORDER": "LOFT_SPINE_PROFILES_NOT_IN_ORDER",
            "LOFT_SELECT_SPINE": "LOFT_SELECT_SPINE",
            "DOCUMENT_NOT_FOUND": "DOCUMENT_NOT_FOUND",
            "ELEMENT_NOT_FOUND": "ELEMENT_NOT_FOUND",
            "ENCLOSE_NO_REGION": "ENCLOSE_NO_REGION",
            "ENCLOSE_CANNOT_MERGE_REGIONS": "ENCLOSE_CANNOT_MERGE_REGIONS",
            "ENCLOSE_CANNOT_CREATE_SOLID": "ENCLOSE_CANNOT_CREATE_SOLID",
            "ENCLOSE_NOTHING_SELECTED": "ENCLOSE_NOTHING_SELECTED",
            "ENCLOSE_UNKNOWN_ERROR": "ENCLOSE_UNKNOWN_ERROR",
            "ACCESS_NOT_ALLOWED": "ACCESS_NOT_ALLOWED",
            "LOFT_SPINE_TOO_MANY_GUIDES": "LOFT_SPINE_TOO_MANY_GUIDES",
            "FEATURE_INVALID_NAMESPACE": "FEATURE_INVALID_NAMESPACE",
            "FEATURE_NULL_NOT_ALLOWED": "FEATURE_NULL_NOT_ALLOWED",
            "SKETCH_DIMENSION_LIMIT_ERROR": "SKETCH_DIMENSION_LIMIT_ERROR",
            "FILL_SURFACE_WIRE_CONTINUITY_MISMATCH": "FILL_SURFACE_WIRE_CONTINUITY_MISMATCH",
            "FILL_SURFACE_INTERNAL_CONTINUITY_MISMATCH": "FILL_SURFACE_INTERNAL_CONTINUITY_MISMATCH",
            "LOFT_START_OR_END_CONDITIONS_FAILED": "LOFT_START_OR_END_CONDITIONS_FAILED",
            "LOFT_START_OR_END_CONDITIONS_MAGNITUDE_NO_EFFECT": "LOFT_START_OR_END_CONDITIONS_MAGNITUDE_NO_EFFECT",
            "FOLLOW_LEADER_IS_IN_UNFOLLOWABLE_TAB": "FOLLOW_LEADER_IS_IN_UNFOLLOWABLE_TAB",
            "LOFT_START_OR_END_CONDITIONS_WITH_GUIDES_FAILED": "LOFT_START_OR_END_CONDITIONS_WITH_GUIDES_FAILED",
            "REST_ASSEMBLY_MISSING_TRANSFORM_GROUPS": "REST_ASSEMBLY_MISSING_TRANSFORM_GROUPS",
            "REST_ASSEMBLY_NULL_TRANSFORM_GROUP": "REST_ASSEMBLY_NULL_TRANSFORM_GROUP",
            "REST_ASSEMBLY_NULL_TRANSFORM_GROUP_INSTANCES": "REST_ASSEMBLY_NULL_TRANSFORM_GROUP_INSTANCES",
            "REST_ASSEMBLY_NULL_TRANSFORM_GROUP_INSTANCE": "REST_ASSEMBLY_NULL_TRANSFORM_GROUP_INSTANCE",
            "LOFT_NO_FACE_FOR_GUIDE_CLAMP": "LOFT_NO_FACE_FOR_GUIDE_CLAMP",
            "LOFT_NO_CONTINUITY_CONDITION_AT_INTERNAL_GUIDE": "LOFT_NO_CONTINUITY_CONDITION_AT_INTERNAL_GUIDE",
            "SKETCH_CONSTRAINT_NOT_FOUND": "SKETCH_CONSTRAINT_NOT_FOUND",
            "SKETCH_CONSTRAINT_PARAMETER_NOT_FOUND": "SKETCH_CONSTRAINT_PARAMETER_NOT_FOUND",
            "SKETCH_ENTITY_NOT_FOUND": "SKETCH_ENTITY_NOT_FOUND",
            "SKETCH_ENTITY_PARAMETER_NOT_FOUND": "SKETCH_ENTITY_PARAMETER_NOT_FOUND",
            "FEATURE_PARAMETER_TYPE_MISMATCH": "FEATURE_PARAMETER_TYPE_MISMATCH",
            "FEATURE_CONFIGURED_PARAMETER_NO_VALUES": "FEATURE_CONFIGURED_PARAMETER_NO_VALUES",
            "FEATURE_CONFIGURED_PARAMETER_INCONSISTENT_TYPES": "FEATURE_CONFIGURED_PARAMETER_INCONSISTENT_TYPES",
            "SHEET_METAL_FLANGE_FAIL_AUTO_MITER": "SHEET_METAL_FLANGE_FAIL_AUTO_MITER",
            "SHEET_METAL_FLANGE_FAIL_PARALLEL_EDGE": "SHEET_METAL_FLANGE_FAIL_PARALLEL_EDGE",
            "SHEET_METAL_FLANGE_FAIL_PARALLEL_PLANE": "SHEET_METAL_FLANGE_FAIL_PARALLEL_PLANE",
            "SHEET_METAL_FLANGE_FAIL_PARALLEL_DIRECTION": "SHEET_METAL_FLANGE_FAIL_PARALLEL_DIRECTION",
            "SHEET_METAL_FLANGE_FAIL_NO_BEND": "SHEET_METAL_FLANGE_FAIL_NO_BEND",
            "SHEET_METAL_FLANGE_NO_PARALLEL_ENTITY": "SHEET_METAL_FLANGE_NO_PARALLEL_ENTITY",
            "SHEET_METAL_FLANGE_NO_DIRECTION_ENTITY": "SHEET_METAL_FLANGE_NO_DIRECTION_ENTITY",
            "SHEET_METAL_TAB_NO_BEND": "SHEET_METAL_TAB_NO_BEND",
            "BOOLEAN_NO_SURFACE_IN_MERGE_SCOPE": "BOOLEAN_NO_SURFACE_IN_MERGE_SCOPE",
            "BOOLEAN_NO_SHARED_EDGE_WITH_SURFACE_IN_MERGE_SCOPE": "BOOLEAN_NO_SHARED_EDGE_WITH_SURFACE_IN_MERGE_SCOPE",
            "SHEET_METAL_BEND_RELIEF_NO_CORNER": "SHEET_METAL_BEND_RELIEF_NO_CORNER",
            "SHEET_METAL_CORNER_SELECT_ENTITIES": "SHEET_METAL_CORNER_SELECT_ENTITIES",
            "SHEET_METAL_BEND_RELIEF_SELECT_ENTITIES": "SHEET_METAL_BEND_RELIEF_SELECT_ENTITIES",
            "VRFILLET_RADIUS_REQUIRED_AT_VERTEX": "VRFILLET_RADIUS_REQUIRED_AT_VERTEX",
            "VRFILLET_RHO_REQUIRED_AT_VERTEX": "VRFILLET_RHO_REQUIRED_AT_VERTEX",
            "VRFILLET_MAG_REQUIRED_AT_VERTEX": "VRFILLET_MAG_REQUIRED_AT_VERTEX",
            "VRFILLET_SELECT_VERTICES": "VRFILLET_SELECT_VERTICES",
            "VRFILLET_VERTEX_NOT_ON_CHAIN": "VRFILLET_VERTEX_NOT_ON_CHAIN",
            "VRFILLET_INVALID_CHAIN": "VRFILLET_INVALID_CHAIN",
            "SHEET_METAL_TAB_NO_MERGE": "SHEET_METAL_TAB_NO_MERGE",
            "SHEET_METAL_TAB_NONPLANAR": "SHEET_METAL_TAB_NONPLANAR",
            "SHEET_METAL_TAB_NO_WALL": "SHEET_METAL_TAB_NO_WALL",
            "SHEET_METAL_TAB_NO_TAB": "SHEET_METAL_TAB_NO_TAB",
            "VRFILLET_NO_EFFECT": "VRFILLET_NO_EFFECT",
            "VRFILLET_MULTI_SELECTION": "VRFILLET_MULTI_SELECTION",
            "SHEET_METAL_TAB_NO_PARALLEL_WALL": "SHEET_METAL_TAB_NO_PARALLEL_WALL",
            "SHEET_METAL_CORNER_UNDER_SIZED": "SHEET_METAL_CORNER_UNDER_SIZED",
            "FILLET_PARTIAL_FAIL": "FILLET_PARTIAL_FAIL",
            "VRFILLET_INTERNAL_ZERO": "VRFILLET_INTERNAL_ZERO",
            "SHEET_METAL_TAB_NO_EFFECT": "SHEET_METAL_TAB_NO_EFFECT",
            "SHEET_METAL_TAB_LOW_CLEARANCE": "SHEET_METAL_TAB_LOW_CLEARANCE",
            "SHEET_METAL_TAB_FAILS_MERGE": "SHEET_METAL_TAB_FAILS_MERGE",
            "SHEET_METAL_TAB_COLLISION": "SHEET_METAL_TAB_COLLISION",
            "REPLACE_FACE_FACE_COUNT_CHANGED": "REPLACE_FACE_FACE_COUNT_CHANGED",
            "FACE_OVERLAP": "FACE_OVERLAP",
            "FACE_REMOVED": "FACE_REMOVED",
            "INTERSECTING_EDGES": "INTERSECTING_EDGES",
            "CANNOT_SPLIT_FACE": "CANNOT_SPLIT_FACE",
            "FILL_SURFACE_BAD_SUPPORT": "FILL_SURFACE_BAD_SUPPORT",
            "FILL_SURFACE_SUPPORT_NOT_ON_BOUNDARY": "FILL_SURFACE_SUPPORT_NOT_ON_BOUNDARY",
            "FILL_SURFACE_SUPPORT_NOT_SMOOTH": "FILL_SURFACE_SUPPORT_NOT_SMOOTH",
            "FILL_CURVE_OR_POINT_CONSTRAINTS": "FILL_CURVE_OR_POINT_CONSTRAINTS",
            "SHEET_METAL_ADD_WRONG_MODEL": "SHEET_METAL_ADD_WRONG_MODEL",
            "SHEET_METAL_PATTERN_DISABLED_BOOLEANS": "SHEET_METAL_PATTERN_DISABLED_BOOLEANS",
            "DRAWING_ALL_INSTANCES_HIDDEN": "DRAWING_ALL_INSTANCES_HIDDEN",
            "FILL_SUPPORT_NOT_SMOOTH_INTERNAL": "FILL_SUPPORT_NOT_SMOOTH_INTERNAL",
            "SKETCH_CONSTRAINT_WRONG_SHEET_METAL_BODY": "SKETCH_CONSTRAINT_WRONG_SHEET_METAL_BODY",
            "SKETCH_CONSTRAINT_FLAT_IN_3D": "SKETCH_CONSTRAINT_FLAT_IN_3D",
            "SKETCH_CONSTRAINT_3D_IN_FLAT": "SKETCH_CONSTRAINT_3D_IN_FLAT",
            "FIT_SPLINE_CURVATURE_FACE": "FIT_SPLINE_CURVATURE_FACE",
            "FIT_SPLINE_CANNOT_EVALUATE_CURVATURE_END_CONDITION": "FIT_SPLINE_CANNOT_EVALUATE_CURVATURE_END_CONDITION",
            "FIT_SPLINE_NEED_DIRECTION_FOR_CURVATURE": "FIT_SPLINE_NEED_DIRECTION_FOR_CURVATURE",
            "CONFIGURATION_HAS_BAD_PARAMETERS": "CONFIGURATION_HAS_BAD_PARAMETERS",
            "CONFIGURATION_HAS_BAD_CURRENT_CONFIGURATION": "CONFIGURATION_HAS_BAD_CURRENT_CONFIGURATION",
            "FEATURE_CONFIGURED_PARAMETER_VALUES_HAVE_IDS": "FEATURE_CONFIGURED_PARAMETER_VALUES_HAVE_IDS",
            "WORKSPACE_NO_LONGER_EXISTS": "WORKSPACE_NO_LONGER_EXISTS",
            "CONTENT_STACKING_INVALID_MODE": "CONTENT_STACKING_INVALID_MODE",
            "CONTENT_STACKING_INVALID_COMPONENTS": "CONTENT_STACKING_INVALID_COMPONENTS",
            "CONTENT_STACKING_INVALID_TOP_STACK": "CONTENT_STACKING_INVALID_TOP_STACK",
            "SHEET_METAL_FACE_PATTERN_NO_JOINT": "SHEET_METAL_FACE_PATTERN_NO_JOINT",
            "SHEET_METAL_FACE_PATTERN_FLOATING_CUT": "SHEET_METAL_FACE_PATTERN_FLOATING_CUT",
            "SHEET_METAL_FACE_PATTERN_FLOATING_WALL": "SHEET_METAL_FACE_PATTERN_FLOATING_WALL",
            "SM_FLAT_OP_NO_INTERSECT": "SM_FLAT_OP_NO_INTERSECT",
            "SM_FLAT_OP_PARTIAL_INTERSECT": "SM_FLAT_OP_PARTIAL_INTERSECT",
            "SM_FLAT_OP_ADD_CROSSES_EDGE": "SM_FLAT_OP_ADD_CROSSES_EDGE",
            "SM_FLAT_OP_NON_PLANAR_TOOL": "SM_FLAT_OP_NON_PLANAR_TOOL",
            "SM_FLAT_OP_NON_PLANAR_TARGET": "SM_FLAT_OP_NON_PLANAR_TARGET",
            "SM_FLAT_OPERATION_FAILED": "SM_FLAT_OPERATION_FAILED",
            "SM_FLAT_OP_LEGACY_MODEL": "SM_FLAT_OP_LEGACY_MODEL",
            "SHEET_METAL_FLANGE_NON_LINEAR_EDGES": "SHEET_METAL_FLANGE_NON_LINEAR_EDGES",
            "SHEET_METAL_CYLINDER_BEND": "SHEET_METAL_CYLINDER_BEND",
            "SHEET_METAL_INVALID_FACE": "SHEET_METAL_INVALID_FACE",
            "SHEET_METAL_ROLLED_CORNER_RELIF": "SHEET_METAL_ROLLED_CORNER_RELIF",
            "SHEET_METAL_RELIEF_FAILURES": "SHEET_METAL_RELIEF_FAILURES",
            "EDGE_CHANGE_FAILED": "EDGE_CHANGE_FAILED",
            "BOOLEAN_INPUTS_NOT_SOLID": "BOOLEAN_INPUTS_NOT_SOLID",
            "TRANSFORM_SCALE_SELECTION": "TRANSFORM_SCALE_SELECTION",
            "SHEET_METAL_SUBTRACT_DESTROYS_SHEET": "SHEET_METAL_SUBTRACT_DESTROYS_SHEET",
            "SPLIT_KEEP_TOOLS_WITH_FACE": "SPLIT_KEEP_TOOLS_WITH_FACE",
            "SPLIT_TRIM_WITH_SINGLE_FACE": "SPLIT_TRIM_WITH_SINGLE_FACE",
            "SHEET_METAL_SKETCH_DETACHED_FACE": "SHEET_METAL_SKETCH_DETACHED_FACE",
            "HOLE_CUT_FAIL": "HOLE_CUT_FAIL",
            "SHEET_METAL_FLAT_OP_ROLL_FAIL": "SHEET_METAL_FLAT_OP_ROLL_FAIL",
            "PATTERN_EDGE_FAILED": "PATTERN_EDGE_FAILED",
            "SHEET_METAL_FACE_PATTERN_NO_VERTEX": "SHEET_METAL_FACE_PATTERN_NO_VERTEX",
            "SHEET_METAL_FACE_PATTERN_PARTIAL_FLOATING_WALL": "SHEET_METAL_FACE_PATTERN_PARTIAL_FLOATING_WALL",
            "EXTRUDE_3D_AND_FLAT": "EXTRUDE_3D_AND_FLAT",
            "PATTERN_SWITCH_TO_PER_INSTANCE": "PATTERN_SWITCH_TO_PER_INSTANCE",
            "PATTERN_NO_GEOM_FROM_FEATURES": "PATTERN_NO_GEOM_FROM_FEATURES",
            "RM_NO_LINK_PERMISSION_TO_REVISION": "RM_NO_LINK_PERMISSION_TO_REVISION",
            "DRAFT_SELECT_PARTING_EDGES": "DRAFT_SELECT_PARTING_EDGES",
            "DRAFT_CONFLICTING_OPPOSITION": "DRAFT_CONFLICTING_OPPOSITION",
            "SPLIT_SELECT_TARGETS": "SPLIT_SELECT_TARGETS",
            "SPLIT_SELECT_TOOL": "SPLIT_SELECT_TOOL",
            "SPLIT_SELECT_FACE_TARGETS": "SPLIT_SELECT_FACE_TARGETS",
            "SPLIT_SELECT_FACE_TOOLS": "SPLIT_SELECT_FACE_TOOLS",
            "ASSEMBLY_PATTERN_EXCEED_MAX_INSTANCE_COUNT": "ASSEMBLY_PATTERN_EXCEED_MAX_INSTANCE_COUNT",
            "ASSEMBLY_PATTERN_RECURSIVE_SEED": "ASSEMBLY_PATTERN_RECURSIVE_SEED",
            "NO_LINK_PERMISSION_TO_THIS_DOCUMENT": "NO_LINK_PERMISSION_TO_THIS_DOCUMENT",
            "SM_FLAT_OP_FACES_NOT_COPLANAR": "SM_FLAT_OP_FACES_NOT_COPLANAR",
            "FLATTENED_SHEET_METAL_SKETCH_PROHIBTED": "FLATTENED_SHEET_METAL_SKETCH_PROHIBTED",
            "DRAFT_SELECT_PULL_DIRECTION_ENTITY": "DRAFT_SELECT_PULL_DIRECTION_ENTITY",
            "DRAFT_PARALLEL_PARTING_EDGE": "DRAFT_PARALLEL_PARTING_EDGE",
            "FLATTENED_SHEET_METAL_SKETCH_ONE_FACE": "FLATTENED_SHEET_METAL_SKETCH_ONE_FACE",
            "SKETCH_OFFSET_CHAIN_FAIL": "SKETCH_OFFSET_CHAIN_FAIL",
            "SKETCH_OFFSET_BASE_CURVE_CONSUMED": "SKETCH_OFFSET_BASE_CURVE_CONSUMED",
            "SKETCH_OFFSET_SPLINE_SPLIT": "SKETCH_OFFSET_SPLINE_SPLIT",
            "SKETCH_OFFSET_ELLIPSE_SPLIT": "SKETCH_OFFSET_ELLIPSE_SPLIT",
            "CANNOT_FIND_FLATTENED_BODY": "CANNOT_FIND_FLATTENED_BODY",
            "SHEET_METAL_ACTIVE_MODEL_REQUIRED": "SHEET_METAL_ACTIVE_MODEL_REQUIRED",
            "SKETCH_REFERENCE_WRONG_FLAT_PART": "SKETCH_REFERENCE_WRONG_FLAT_PART",
            "SKETCH_CANNOT_REFERENCE_3D": "SKETCH_CANNOT_REFERENCE_3D",
            "SM_FLAT_OP_CANNOT_JOIN_DIFFERENT_TRANSFORMS": "SM_FLAT_OP_CANNOT_JOIN_DIFFERENT_TRANSFORMS",
            "DEFINED_IN_3D_CANT_REFERENCE_SM_FLAT": "DEFINED_IN_3D_CANT_REFERENCE_SM_FLAT",
            "DEFINED_IN_SM_FLAT_CANT_REFERENCE_3D": "DEFINED_IN_SM_FLAT_CANT_REFERENCE_3D",
            "CANNOT_CHANGE_REFERENCE_TO_DELETED_DOCUMENT": "CANNOT_CHANGE_REFERENCE_TO_DELETED_DOCUMENT",
            "COPIED_SKETCH_NOT_FOUND": "COPIED_SKETCH_NOT_FOUND",
            "VARIABLE_NAME_TOO_LONG": "VARIABLE_NAME_TOO_LONG",
            "NO_INSTANCE_TO_REPLACE": "NO_INSTANCE_TO_REPLACE",
            "ASSEMBLY_REPLACE_NO_REPLACER": "ASSEMBLY_REPLACE_NO_REPLACER",
            "CANNOT_REPLACE_IN_SUBASSEMBLIES": "CANNOT_REPLACE_IN_SUBASSEMBLIES",
            "FILL_SURFACE_G1_FAIL": "FILL_SURFACE_G1_FAIL",
            "ALL_CONFIGURATION_PARAMETERS_NOT_SPECIFIED": "ALL_CONFIGURATION_PARAMETERS_NOT_SPECIFIED",
            "ASSEMBLY_REPLACE_INVALID_SEED_INSTANCE": "ASSEMBLY_REPLACE_INVALID_SEED_INSTANCE",
            "ASSEMBLY_REPLACE_INVALID_SEED_TYPE": "ASSEMBLY_REPLACE_INVALID_SEED_TYPE",
            "ASSEMBLY_REPLACE_SUPPRESSED_FORBIDDEN": "ASSEMBLY_REPLACE_SUPPRESSED_FORBIDDEN",
            "DISPLAY_STATES_INVALID_NAME": "DISPLAY_STATES_INVALID_NAME",
            "DISPLAY_STATES_DUPLICATE_NAME": "DISPLAY_STATES_DUPLICATE_NAME",
            "WORKSPACE_UPGRADE_NONE_ENABLE_CAPABILITY": "WORKSPACE_UPGRADE_NONE_ENABLE_CAPABILITY",
            "DISPLAY_STATES_UPDATE_HAD_NO_EFFECT": "DISPLAY_STATES_UPDATE_HAD_NO_EFFECT",
            "DISPLAY_STATES_APPLY_HAD_NO_EFFECT": "DISPLAY_STATES_APPLY_HAD_NO_EFFECT",
            "DISPLAY_STATES_DOES_NOT_EXIST": "DISPLAY_STATES_DOES_NOT_EXIST",
            "LOFT_SPINE_NONTANGENT_PATH": "LOFT_SPINE_NONTANGENT_PATH",
            "LOFT_SPINE_APPROXIMATED": "LOFT_SPINE_APPROXIMATED",
            "REST_ASSEMBLY_REVISION_PART_NUMBER_BLANK": "REST_ASSEMBLY_REVISION_PART_NUMBER_BLANK",
            "REST_ASSEMBLY_REVISION_NOT_FOUND": "REST_ASSEMBLY_REVISION_NOT_FOUND",
            "ROLL_FAILED": "ROLL_FAILED",
            "ROLL_CANNOT_ADJUST": "ROLL_CANNOT_ADJUST",
            "ROLL_MISMATCHED_SOURCE": "ROLL_MISMATCHED_SOURCE",
            "INVALID_ROLL_SURFACE": "INVALID_ROLL_SURFACE",
            "ROLL_NOT_TO_OR_FROM_PLANE": "ROLL_NOT_TO_OR_FROM_PLANE",
            "FAILED_TO_FIND_ELEMENT_FOR_TO_STATE": "FAILED_TO_FIND_ELEMENT_FOR_TO_STATE",
            "FAILED_TO_FIND_VERSION_FOR_TO_STATE": "FAILED_TO_FIND_VERSION_FOR_TO_STATE",
            "MISSING_REVISION_ID_FOR_TO_STATE": "MISSING_REVISION_ID_FOR_TO_STATE",
            "MISSING_VERSION_ID_FOR_TO_STATE": "MISSING_VERSION_ID_FOR_TO_STATE",
            "INVALID_DOCUMENT_ID_FOR_TO_STATE": "INVALID_DOCUMENT_ID_FOR_TO_STATE",
            "ASSEMBLY_CONTAINS_MISSING_PART_DATA": "ASSEMBLY_CONTAINS_MISSING_PART_DATA",
            "SHEET_METAL_HEM_NO_EDGES": "SHEET_METAL_HEM_NO_EDGES",
            "SHEET_METAL_HEM_NON_LINEAR_EDGES": "SHEET_METAL_HEM_NON_LINEAR_EDGES",
            "SHEET_METAL_HEM_FAIL_ALIGNMENT": "SHEET_METAL_HEM_FAIL_ALIGNMENT",
            "SHEET_METAL_HEM_TOO_SHORT": "SHEET_METAL_HEM_TOO_SHORT",
            "SHEET_METAL_HEM_TEAR_DROP_GAP_TOO_LARGE": "SHEET_METAL_HEM_TEAR_DROP_GAP_TOO_LARGE",
            "SHEET_METAL_HEM_INTERNAL": "SHEET_METAL_HEM_INTERNAL",
            "SHEET_METAL_HEM_FAILED": "SHEET_METAL_HEM_FAILED",
            "INVALID_CONFIGURATION_SPECIFIED": "INVALID_CONFIGURATION_SPECIFIED",
            "EDGEBLEND_PARTIAL": "EDGEBLEND_PARTIAL",
            "CHAMFER_PARTIAL_FAIL": "CHAMFER_PARTIAL_FAIL",
            "LOFT_INCONSISTENT_EXTENT": "LOFT_INCONSISTENT_EXTENT",
            "LOFT_CANT_TRIM_PROFILES_INFO": "LOFT_CANT_TRIM_PROFILES_INFO",
            "LOFT_CANT_TRIM_GUIDES_INFO": "LOFT_CANT_TRIM_GUIDES_INFO",
            "NON_GEOMETRIC_ITEM_INSERTION_FAILED": "NON_GEOMETRIC_ITEM_INSERTION_FAILED",
            "SHEET_METAL_FLANGE_NEXT_TO_CYLINDER_BEND": "SHEET_METAL_FLANGE_NEXT_TO_CYLINDER_BEND",
            "SHEET_METAL_HEM_NEXT_TO_CYLINDER_BEND": "SHEET_METAL_HEM_NEXT_TO_CYLINDER_BEND",
            "SHEET_METAL_MOVE_FACE_NEXT_TO_CYLINDER_BEND": "SHEET_METAL_MOVE_FACE_NEXT_TO_CYLINDER_BEND",
            "ASSEMBLY_UNKNOWN_SOLVE_ERROR": "ASSEMBLY_UNKNOWN_SOLVE_ERROR",
            "ASSEMBLY_FEATURE_FAILED_TO_RESOLVE_ALL_INSTANCES": "ASSEMBLY_FEATURE_FAILED_TO_RESOLVE_ALL_INSTANCES",
            "SHEET_METAL_ORPHANED_BEND": "SHEET_METAL_ORPHANED_BEND",
            "ASSEMBLY_CONTEXT_INVALID_REFERENCE": "ASSEMBLY_CONTEXT_INVALID_REFERENCE",
            "ASSEMBLY_FEATURE_QUERY_DATA_MISSING": "ASSEMBLY_FEATURE_QUERY_DATA_MISSING",
            "NON_GEOMETRIC_ITEMS_DOCUMENT_NOT_OWNED_BY_COMPANY": "NON_GEOMETRIC_ITEMS_DOCUMENT_NOT_OWNED_BY_COMPANY",
            "NON_GEOMETRIC_ITEM_NO_ITEM_FOUND_FOR_DOCUMENT_OWNER": "NON_GEOMETRIC_ITEM_NO_ITEM_FOUND_FOR_DOCUMENT_OWNER",
            "EXTEND_SHEET_BODY_NO_TARGET": "EXTEND_SHEET_BODY_NO_TARGET",
            "EXTEND_TARGET_MISSED": "EXTEND_TARGET_MISSED",
            "EXTEND_SELF_INTERSECTION": "EXTEND_SELF_INTERSECTION",
            "EXTEND_FAILED": "EXTEND_FAILED",
            "EXTEND_TO_FACE_FAILED": "EXTEND_TO_FACE_FAILED",
            "TRIM_TO_MULTI_FAILED": "TRIM_TO_MULTI_FAILED",
            "EXTEND_NON_LAMINAR": "EXTEND_NON_LAMINAR",
            "EXTEND_TO_VERTEX_FAILED": "EXTEND_TO_VERTEX_FAILED",
            "TRIM_FAILED": "TRIM_FAILED",
            "SKETCH_EQUAL_CURVATURE_FAILED": "SKETCH_EQUAL_CURVATURE_FAILED",
            "SKETCH_EQUAL_CURVATURE_INPUT_ERROR": "SKETCH_EQUAL_CURVATURE_INPUT_ERROR",
            "ASSEMBLY_EXPLODE_STEP_DIRECTION_QUERY_FAILED": "ASSEMBLY_EXPLODE_STEP_DIRECTION_QUERY_FAILED",
            "WRAP_DESTINATION_NO_FACE": "WRAP_DESTINATION_NO_FACE",
            "WRAP_TRIM_FAILED": "WRAP_TRIM_FAILED",
            "WRAP_TRIM_NO_INTERSECTION": "WRAP_TRIM_NO_INTERSECTION",
            "WRAP_IMPRINT_FAILED": "WRAP_IMPRINT_FAILED",
            "WRAP_NEEDS_ANCHOR": "WRAP_NEEDS_ANCHOR",
            "WRAP_SELECT_TOOLS": "WRAP_SELECT_TOOLS",
            "WRAP_SELECT_TARGET": "WRAP_SELECT_TARGET",
            "CREATE_COMPOSITE_PART_FAILED": "CREATE_COMPOSITE_PART_FAILED",
            "NESTED_COMPOSITE_PARTS": "NESTED_COMPOSITE_PARTS",
            "ASSEMBLY_EXPLOSION_DOES_NOT_EXIST": "ASSEMBLY_EXPLOSION_DOES_NOT_EXIST",
            "FILL_SURFACE_NO_GUIDES": "FILL_SURFACE_NO_GUIDES",
            "HOLE_INCONSISTENT_TAP_INFO": "HOLE_INCONSISTENT_TAP_INFO",
            "WRAP_SOURCE_DIFFERING_PLANES": "WRAP_SOURCE_DIFFERING_PLANES",
            "WRAP_IMPRINT_SHEET_METAL": "WRAP_IMPRINT_SHEET_METAL",
            "WRAP_SELECT_ANCHORS": "WRAP_SELECT_ANCHORS",
            "OPERATION_CANCELLED": "OPERATION_CANCELLED",
            "COMPOSITE_CLOSED_OVERLAP": "COMPOSITE_CLOSED_OVERLAP",
            "COMPOSITE_OPEN_OVERLAP": "COMPOSITE_OPEN_OVERLAP",
            "COMPOSITE_SKETCH": "COMPOSITE_SKETCH",
            "COMPOSITE_MATE_CONNECTOR": "COMPOSITE_MATE_CONNECTOR",
            "COMPOSITE_PLANE": "COMPOSITE_PLANE",
            "COMPOSITE_NON_MODIFIABLE": "COMPOSITE_NON_MODIFIABLE",
            "ASSEMBLY_REPLACE_WILL_CAUSE_CYCLES": "ASSEMBLY_REPLACE_WILL_CAUSE_CYCLES",
            "CREATE_OUTLINE_TWO_OFFSET_FACES_PER_BODY": "CREATE_OUTLINE_TWO_OFFSET_FACES_PER_BODY",
            "SHEET_METAL_TOOL_DOES_NOT_CUT_THROUGH": "SHEET_METAL_TOOL_DOES_NOT_CUT_THROUGH",
            "PERIODIC_BSPLINESURFACE_NOT_CLOSED": "PERIODIC_BSPLINESURFACE_NOT_CLOSED",
            "PERIODIC_BSPLINESURFACE_NOT_SMOOTH": "PERIODIC_BSPLINESURFACE_NOT_SMOOTH",
            "CANNOT_MAKE_BSPLINESURFACE": "CANNOT_MAKE_BSPLINESURFACE",
            "BSPLINESURFACE_NOT_G1": "BSPLINESURFACE_NOT_G1",
            "BAD_2D_BOUNDARY_BSPLINECURVE_DEFINITION": "BAD_2D_BOUNDARY_BSPLINECURVE_DEFINITION",
            "PERIODIC_2D_BOUNDARY_BSPLINECURVE_NOT_CLOSED": "PERIODIC_2D_BOUNDARY_BSPLINECURVE_NOT_CLOSED",
            "PERIODIC_2D_BOUNDARY_BSPLINECURVE_NOT_SMOOTH": "PERIODIC_2D_BOUNDARY_BSPLINECURVE_NOT_SMOOTH",
            "BSPLINESURFACE_BOUNDARY_NOT_SINGLE_CLOSED_LOOP": "BSPLINESURFACE_BOUNDARY_NOT_SINGLE_CLOSED_LOOP",
            "BSPLINESURFACE_FAILED_TO_MAKE_SOLID": "BSPLINESURFACE_FAILED_TO_MAKE_SOLID",
            "BOOLEAN_NO_SHARED_EDGE_WITH_OTHER_SURFACE": "BOOLEAN_NO_SHARED_EDGE_WITH_OTHER_SURFACE",
            "BOOLEAN_NO_SURFACE_TO_MERGE_WITH": "BOOLEAN_NO_SURFACE_TO_MERGE_WITH",
            "BOOLEAN_INVALID_PARAMETER_COMBINATION": "BOOLEAN_INVALID_PARAMETER_COMBINATION",
            "MODIFY_COMPOSITE_PART_FAILED": "MODIFY_COMPOSITE_PART_FAILED",
            "INSTANCE_CONSUMED_BY_CLOSED_COMPOSITE": "INSTANCE_CONSUMED_BY_CLOSED_COMPOSITE",
            "EXPLODE_STEP_INSTANCE_IS_SUPPRESSED": "EXPLODE_STEP_INSTANCE_IS_SUPPRESSED",
            "NO_COMPOSITE_FOR_SINGLE_SOLID": "NO_COMPOSITE_FOR_SINGLE_SOLID",
            "COMPOSITE_PART_ADD_AND_REMOVE_BODY": "COMPOSITE_PART_ADD_AND_REMOVE_BODY",
            "BOOLEAN_TOOL_INPUTS_NOT_SOLID": "BOOLEAN_TOOL_INPUTS_NOT_SOLID",
            "BOOLEAN_CANNOT_MIX_SOLIDS_AND_SURFACES": "BOOLEAN_CANNOT_MIX_SOLIDS_AND_SURFACES",
            "BSPLINESURFACE_CONTROL_POINTS_GRID_TOO_LONG": "BSPLINESURFACE_CONTROL_POINTS_GRID_TOO_LONG",
            "BSPLINE_TOO_MANY_CONTROL_POINTS": "BSPLINE_TOO_MANY_CONTROL_POINTS",
            "BSPLINE_DEGREE_TOO_HIGH": "BSPLINE_DEGREE_TOO_HIGH",
            "COMPOSITE_PART_SELECT_ENTITIES": "COMPOSITE_PART_SELECT_ENTITIES",
            "DELETE_COMPOSITE_PART_FAILED": "DELETE_COMPOSITE_PART_FAILED",
            "SURFACES_NOT_SUPPORTED_BY_PATTERN_REMOVE_AND_INTERSECT": "SURFACES_NOT_SUPPORTED_BY_PATTERN_REMOVE_AND_INTERSECT",
            "SHEET_METAL_TAB_MERGE_AND_SUBTRACT_SAME_FLANGE": "SHEET_METAL_TAB_MERGE_AND_SUBTRACT_SAME_FLANGE",
            "EXPLODED_VIEW_INVALID_STARTING_POSITION": "EXPLODED_VIEW_INVALID_STARTING_POSITION",
            "CANNOT_EXPLODE_SKETCH": "CANNOT_EXPLODE_SKETCH",
            "FAILED_TO_CREATE_EXPLODED_VIEW": "FAILED_TO_CREATE_EXPLODED_VIEW",
            "UNKNOWN": "UNKNOWN",
        },
    }

    validations = {}

    additional_properties_type = None

    @staticmethod
    def openapi_types():
        """
        This must be a class method so a model may have properties that are
        of type self, this ensures that we don't create a cyclic import

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            "bodies": (
                [bt_export_tessellated_body3398.BTExportTessellatedBody3398],
            ),  # noqa: E501
            "error_enum": (str,),  # noqa: E501
        }

    @staticmethod
    def discriminator():
        return None

    attribute_map = {
        "bodies": "bodies",  # noqa: E501
        "error_enum": "errorEnum",  # noqa: E501
    }

    @staticmethod
    def _composed_schemas():
        return None

    required_properties = set(
        [
            "_data_store",
            "_check_type",
            "_from_server",
            "_path_to_item",
            "_configuration",
        ]
    )

    def __init__(
        self,
        _check_type=True,
        _from_server=False,
        _path_to_item=(),
        _configuration=None,
        **kwargs
    ):  # noqa: E501
        """bt_export_tessellated_edges_response327.BTExportTessellatedEdgesResponse327 - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _from_server (bool): True if the data is from the server
                                False if the data is from the client (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            bodies ([bt_export_tessellated_body3398.BTExportTessellatedBody3398]): [optional]  # noqa: E501
            error_enum (str): [optional]  # noqa: E501
        """

        self._data_store = {}
        self._check_type = _check_type
        self._from_server = _from_server
        self._path_to_item = _path_to_item
        self._configuration = _configuration

        for var_name, var_value in six.iteritems(kwargs):
            if (
                var_name not in self.attribute_map
                and self._configuration is not None
                and self._configuration.discard_unknown_keys
                and self.additional_properties_type is None
            ):
                # discard variable.
                continue
            setattr(self, var_name, var_value)



from onshape_client.oas.api_client import ApiClient
from onshape_client.client import Client
import pytest
import json


@pytest.mark.parametrize("element", ["ps_configurable_cube"], indirect=True)
def test_get_custom_call(client, element):
    result = client.api_client.request(
        method="GET",
        url=f"{client.configuration.host}/api/parts/d/{element.did}/{element.wvm}/{element.wvmid}/e/{element.eid}",
        headers={
            "Accept": "application/vnd.onshape.v1+json;charset=utf-8;qs=0.1, application/json;charset=utf-8; qs=0.09"
        },
        query_params={},
    )
    result = json.loads(result.data)
    assert result[0]["partId"] == "JHD"



from onshape_client.oas import BTTranslateFormatParams


def test_import_then_drawing_workflow(
    client, new_document, translated_geometry, tmp_dir
):
    imported_element = new_document.import_file(
        file_path=translated_geometry / "Cube.x_t"
    )
    drawing = imported_element.create_drawing()
    drawing.export_file(
        tmp_dir / "test_import_then_drawing_workflow.pdf",
        bt_translate_format_params=BTTranslateFormatParams(format_name="PDF"),
    )



from __future__ import print_function

import json

import pint
import pytest
from onshape_client.assembly import AssemblyDefinition
from onshape_client.onshape_url import OnshapeElement, ConfiguredOnshapeElement
from onshape_client.units import u

ureg = pint.UnitRegistry()


def test_creation_from_urls():
    assert (
        OnshapeElement(
            "https://cad.onshape.com/documents/c8f8013d34183b1de74fa930/w/574b77701d8b74987c273500/e/455ef770951fe37de0b8ff08?configuration=List_TOpkWtvolR0KY4%3Dewf"
        ).configuration
        == "List_TOpkWtvolR0KY4=ewf"
    )


def test_assembly_structural_equivalence(json_assets):
    asm1 = AssemblyDefinition(
        json.load((json_assets / "assembly_definition_three_axes").open())
    )
    asm2 = AssemblyDefinition(
        json.load((json_assets / "assembly_definition_three_axes_copied").open())
    )
    assert asm1.is_structurally_equal_assembly(asm2)


@pytest.mark.parametrize("element", ["ps_configurable_cube"], indirect=True)
def test_configuration_update(element):
    element = ConfiguredOnshapeElement(element.get_url())
    assert element._get_default_configuration_map() == {
        "size": 500 * u.inch,
        "edge_configuration": "Default",
        "show_cube": True,
    }
    element.update_current_configuration({"size": 20 * ureg.m})
    assert (
        element.get_url_with_configuration()
        == "https://cad.onshape.com/documents/cca81d10f239db0db9481e6f/v/6ccf88eb92d55be180c06cf9/e/69c9eedda86512966b20bc90?configuration=edge_configuration%3DDefault%3Bshow_cube%3DTrue%3Bsize%3D20%2Bmeter"
    )
    element.update_current_configuration(
        {"size": 40 * u.inch, "edge_configuration": "chamfered"}
    )
    assert (
        element.get_url_with_configuration()
        == "https://cad.onshape.com/documents/cca81d10f239db0db9481e6f/v/6ccf88eb92d55be180c06cf9/e/69c9eedda86512966b20bc90?configuration=edge_configuration%3Dchamfered%3Bshow_cube%3DTrue%3Bsize%3D40%2Binch"
    )
    element.update_current_configuration(
        {"size": 10 * u.inch, "edge_configuration": "chamfered", "show_cube": False}
    )
    assert (
        element.get_url_with_configuration()
        == "https://cad.onshape.com/documents/cca81d10f239db0db9481e6f/v/6ccf88eb92d55be180c06cf9/e/69c9eedda86512966b20bc90?configuration=edge_configuration%3Dchamfered%3Bshow_cube%3DFalse%3Bsize%3D10%2Binch"
    )



import json

import pytest
from onshape_client.oas import (
    BTMParameterString149,
    BTMSketch151,
    BTMSketchConstraint2,
    BTMSketchCurveSegment155,
    BTMIndividualQuery138,
    BTMParameterQueryList148,
    BTMSketchPoint158,
    BTCurveGeometryLine117,
    BTMSketchCurve4,
    BTCurveGeometryCircle115,
)
from onshape_client.oas.models.bt_feature_definition_call1406 import (
    BTFeatureDefinitionCall1406,
)
from onshape_client.oas.models.bt_id_translation_params import BTIdTranslationParams
from onshape_client.oas.models.btm_feature134 import BTMFeature134
from onshape_client.oas.models.btm_parameter_enum145 import BTMParameterEnum145
from onshape_client.oas.models.btm_parameter_quantity147 import BTMParameterQuantity147


@pytest.mark.parametrize("element", ["ps_configurable_cube"], indirect=True)
def test_translate_ids(element, client):
    sketch_id = "JCF"
    params = BTIdTranslationParams(
        ids=[sketch_id], source_document_microversion="99097c624be5cd434b03f906"
    )
    result = client.part_studios_api.translate_ids(
        element.did, element.wvm, element.wvmid, element.eid, params
    )
    assert result.ids[0].source == sketch_id
    assert result.ids[0].target[0] == sketch_id


@pytest.mark.skip("Not fully supporting deserialization of the feature tree yet.")
@pytest.mark.parametrize("element", ["ps_configurable_cube"], indirect=True)
def test_get_features(element, client):
    result = client.part_studios_api.get_part_studio_features(
        element.did, element.wvm, element.wvmid, element.eid
    )
    assert result.features[0].name == "Sketch 1"


@pytest.mark.parametrize("element", ["ps_configurable_cube"], indirect=True)
def test_export_stl(element, client):
    result = client.part_studios_api.export_stl1(
        element.did, element.wvm, element.wvmid, element.eid, _preload_content=False
    )
    assert len(result.data) > 1000


def test_insert_point_sketch(client, part_studio):
    PLANE_ID = "JDC"  # The plane deterministic ID for the sketch
    plane_query = BTMParameterQueryList148(
        parameter_id="sketchPlane",
        queries=[BTMIndividualQuery138(deterministic_ids=[PLANE_ID])],
    )
    point = BTMSketchPoint158(
        y=-0.071735, x=-0.0564367610245, is_user_point=True, is_construction=False
    )
    sketch = BTMSketch151(
        entities=[point], name="My New Point", parameters=[plane_query]
    )
    feature_definition = BTFeatureDefinitionCall1406(
        feature=sketch, bt_type="BTFeatureDefinitionCall-1406"
    )
    client.part_studios_api.add_part_studio_feature(
        did=part_studio.did,
        wvm=part_studio.wvm,
        wvmid=part_studio.wvmid,
        eid=part_studio.eid,
        bt_feature_definition_call_1406=feature_definition,
        _preload_content=False,
    )


def test_insert_line_sketch(client, part_studio):
    FEATURE_NAME = "My New Line"
    PLANE_ID = "JDC"  # The plane deterministic ID for the sketch
    plane_query = BTMParameterQueryList148(
        parameter_id="sketchPlane",
        queries=[BTMIndividualQuery138(deterministic_ids=[PLANE_ID])],
    )
    LINE_ID = "myLine"
    START = "start"
    END = "end"
    POINT_ID = "myPoint"

    point_string_param = BTMParameterString149(
        value=POINT_ID, parameter_id="localFirst", bt_type="BTMParameterString-149"
    )
    line_end_string_param = BTMParameterString149(
        value=f"{LINE_ID}.{START}",
        parameter_id="localSecond",
        bt_type="BTMParameterString-149",
    )
    coincident_constraint = BTMSketchConstraint2(
        constraint_type="COINCIDENT",
        parameters=[point_string_param, line_end_string_param],
        entity_id="constrainId",
        bt_type="BTMSketchConstraint-2",
    )
    point = BTMSketchPoint158(
        y=-0.071735,
        x=-0.0564367610245,
        entity_id=POINT_ID,
        bt_type="BTMSketchPoint-158",
    )
    line_geometry = BTCurveGeometryLine117(
        pnt_x=0.0, pnt_y=0.0, dir_x=0.5, dir_y=0.5, bt_type="BTCurveGeometryLine-117"
    )
    line = BTMSketchCurveSegment155(
        start_point_id=f"{LINE_ID}.{START}",
        end_point_id=f"{LINE_ID}.{END}",
        start_param=0.0,
        end_param=1.0,
        geometry=line_geometry,
        entity_id=LINE_ID,
        bt_type="BTMSketchCurveSegment-155",
    )
    sketch = BTMSketch151(
        entities=[line, point],
        name="My New Line",
        parameters=[plane_query],
        constraints=[coincident_constraint],
        bt_type="BTMSketch-151",
    )
    feature_definition = BTFeatureDefinitionCall1406(feature=sketch)
    client.part_studios_api.add_part_studio_feature(
        did=part_studio.did,
        wvm=part_studio.wvm,
        wvmid=part_studio.wvmid,
        eid=part_studio.eid,
        bt_feature_definition_call_1406=feature_definition,
        _preload_content=False,
    )
    features = client.part_studios_api.get_part_studio_features(
        did=part_studio.did,
        wvm=part_studio.wvm,
        wvmid=part_studio.wvmid,
        eid=part_studio.eid,
        _preload_content=False,
    )
    assert json.loads(features.data)["features"][0]["name"] == FEATURE_NAME


def test_create_surface(client, part_studio):
    test_insert_line_sketch(client, part_studio)
    tool_body_type = BTMParameterEnum145(
        value="SURFACE", enum_name="ToolBodyType", parameter_id="bodyType"
    )
    operation_type = BTMParameterEnum145(
        value="NEW",
        enum_name="NewSurfaceOperationType",
        parameter_id="surfaceOperationType",
    )
    line_query = BTMParameterQueryList148(
        parameter_id="surfaceEntities",
        queries=[BTMIndividualQuery138(deterministic_ids=["JFB"])],
    )
    length = BTMParameterQuantity147(expression="3.03*in", parameter_id="depth")
    extrude_feature = BTMFeature134(
        bt_type="BTMFeature-134",
        name="My extrude",
        feature_type="extrude",
        parameters=[tool_body_type, operation_type, line_query, length],
    )
    feature_definition = BTFeatureDefinitionCall1406(feature=extrude_feature)
    client.part_studios_api.add_part_studio_feature(
        did=part_studio.did,
        wvm=part_studio.wvm,
        wvmid=part_studio.wvmid,
        eid=part_studio.eid,
        bt_feature_definition_call_1406=feature_definition,
        _preload_content=False,
    )


def test_create_circle(client, part_studio):
    FEATURE_NAME = "My New Circle"
    PLANE_ID = "JDC"  # The plane deterministic ID for the sketch
    plane_query = BTMParameterQueryList148(
        parameter_id="sketchPlane",
        queries=[BTMIndividualQuery138(deterministic_ids=[PLANE_ID])],
    )
    CIRCLE_ID = "myCircle"

    circle_geometry = BTCurveGeometryCircle115(
        radius=0.5, xcenter=0.0, ycenter=0.0, xdir=1.0, ydir=0.0, clockwise=False
    )
    circle = BTMSketchCurve4(
        center_id=f"{CIRCLE_ID}.center", entity_id=CIRCLE_ID, geometry=circle_geometry
    )
    sketch = BTMSketch151(
        entities=[circle], name=FEATURE_NAME, parameters=[plane_query]
    )
    feature_definition = BTFeatureDefinitionCall1406(
        feature=sketch, bt_type="BTFeatureDefinitionCall-1406"
    )
    client.part_studios_api.add_part_studio_feature(
        did=part_studio.did,
        wvm=part_studio.wvm,
        wvmid=part_studio.wvmid,
        eid=part_studio.eid,
        bt_feature_definition_call_1406=feature_definition,
        _preload_content=False,
    )
